// $Id: TLdaAnalysis.cxx 505 2009-11-05 18:03:50Z goofy $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TLdaAnalysis.h"

#include <stdlib.h>
#include <time.h>
#include "Riostream.h"

#include "TH1.h"
#include "TFile.h"
#include "TSystem.h"

#include "s_filhe_swap.h"
#include "s_bufhe_swap.h"
//#include "f_ut_utime.h"

#include "TGo4AnalysisStep.h"
#include "TGo4StepFactory.h"
#include "TLdaParameters.h"
#include "TLdaUnpackEvent.h"
#include "TLdaAnlEvent.h"

#include "TGo4Version.h"
//#include "TGo4Log.h"
//#include "TGo4MbsEvent.h"




#include "Go4EventServer.h"

//***********************************************************
TLdaAnalysis::TLdaAnalysis() :
   TGo4Analysis(),
   fMbsEvent(0),
   fRawEvent(0),
   fAnlEvent(0)

{
  cout << "Wrong constructor TLdaAnalysis()!" << endl;
}

//***********************************************************
// this constructor is called by go4analysis executable
TLdaAnalysis::TLdaAnalysis(int argc, char** argv) :
   TGo4Analysis(argc, argv),
   fMbsEvent(0),
   fRawEvent(0),
   fAnlEvent(0),
   fSize(0),
   fEvents(0),
   fLastEvent(0)
{
   if (!TGo4Version::CheckVersion(__GO4BUILDVERSION__)) {
      cout << "****  Go4 version mismatch" << endl;
      exit(-1);
   }

   cout << "**** TLdaAnalysis: Create" << endl;
   SetStepChecking(kFALSE); // necessary, as we use non-subsequent steps


   TGo4StepFactory* factory1 = new TGo4StepFactory("UnpackFactory");
   factory1->DefEventProcessor("UnpackProc", "TLdaUnpackProc");// object name, class name
   factory1->DefOutputEvent("UnpackEvent", "TLdaUnpackEvent"); // object name, class name
   TGo4AnalysisStep* step1 = new TGo4AnalysisStep("Unpack",factory1,0,0,0);
   step1->SetErrorStopEnabled(kTRUE);
   AddAnalysisStep(step1);
// These settings will be overwritten by setup.C
   step1->SetSourceEnabled(kTRUE);
   step1->SetStoreEnabled(kFALSE);
   step1->SetProcessEnabled(kTRUE);

// Create step 2 Analysis.
   TGo4StepFactory* factory2 = new TGo4StepFactory("AnalysisFactory");
   factory2->DefInputEvent("UnpackEvent", "TLdaUnpackEvent"); // object name, class name
   factory2->DefEventProcessor("AnlProc", "TLdaAnlProc"); // object name, class name
   factory2->DefOutputEvent("AnlEvent", "TLdaAnlEvent"); // object name, class name
   TGo4AnalysisStep* step2    = new TGo4AnalysisStep("Analysis",factory2,0,0,0);
   step2->SetErrorStopEnabled(kTRUE);
   AddAnalysisStep(step2);
// These settings will be overwritten by setup.C
   step2->SetSourceEnabled(kTRUE);
   step2->SetStoreEnabled(kFALSE);
   step2->SetProcessEnabled(kTRUE);


   //////////////// Parameter //////////////////////////
   // At this point, auto-save file has not yet been read!
   // Therefore parameter values set here will be overwritten
   // if an auto-save file is there.
   fPar = (TLdaParameters *)MakeParameter("TimeParameters","TLdaParameters");

   // This condition is used in both steps.
   // Therfore we create it here
   // fWinCon1 = MakeWinCond("wincon1", 50, 2000);

   //Create all conditions already here ????????
 
   if (!gSystem->AccessPathName("SetParameters.C"))
   gROOT->ProcessLine(".x SetParameters.C");
   else
     cout << "**** Cannot find SetParameters.C script in current directory ! ****" << endl;
   
//    if (!gSystem->AccessPathName("SetConditions.C"))
//   gROOT->ProcessLine(".x SetConditions.C");
//   else
//     cout << "**** Cannot find SetConditions.C script in current directory ! ****" << endl;
}
//***********************************************************
TLdaAnalysis::~TLdaAnalysis()
{
  cout << "**** TLdaAnalysis: Delete" << endl;
}
//***********************************************************

//-----------------------------------------------------------
Int_t TLdaAnalysis::UserPreLoop()
{
  cout << "**** TLdaAnalysis: PreLoop" << endl;
  Print(); // printout the step settings
  cout << "**************************************" << endl;
   // we update the pointers to the current event structures here:
   fMbsEvent = dynamic_cast<TGo4MbsEvent*>    (GetInputEvent("Unpack"));   // of step "Unpack"
   fRawEvent = dynamic_cast<TLdaUnpackEvent*> (GetOutputEvent("Unpack"));


   fEvents=0;
   fLastEvent=0;

   // create histogram for UserEventFunc
   // At this point, the histogram has been restored
   // from auto-save file if any.
   fSize = (TH1D*) MakeTH1('D', "Step0_PreLoop/Eventsize", "Eventsize", 10000,
			0, 10000, "amount of data [Byte] per readout", "occurrences");
   
   fPar->piLaserStep = 0;
   
   return 0;
}
//-----------------------------------------------------------
Int_t TLdaAnalysis::UserPostLoop()
{
  cout << "**** TLdaAnalysis: PostLoop" << endl;
  cout << "Last event: " << fLastEvent << " Total events: " << fEvents << endl;
  if(fMbsEvent)
    {
      // we can check some properties of last event here:
      //fMbsEvent->PrintEvent(); // header and data content

      // fileheader structure:
      s_filhe* fileheader=fMbsEvent->GetMbsSourceHeader();
      if(fileheader)
         {
           cout <<"\nInput file was: "<<fileheader->filhe_file << endl;
           cout <<"Tapelabel:\t" << fileheader->filhe_label<<endl;
           cout <<"UserName:\t" << fileheader->filhe_user<<endl;
           cout <<"RunID:\t" << fileheader->filhe_run<<endl;
           cout <<"\tExplanation: "<<fileheader->filhe_exp <<endl;
           cout <<"\tComments: "<<endl;
           Int_t numlines=fileheader->filhe_lines;
           for(Int_t i=0; i<numlines;++i)
            {
               cout<<"\t\t"<<fileheader->s_strings[i].string << endl;
            }
         }

      // mbs buffer header structure:
      s_bufhe* bufheader=fMbsEvent->GetMbsBufferHeader();
      if(bufheader)
         {
            cout <<"Last Buffer:"<<endl;
            cout <<"\tNumber: "<<bufheader->l_buf << endl;   
            std::cout <<"\ttime lo: "<<bufheader->l_time[0] << std::endl; // seconds since epoch 1970
            std::cout <<"\ttime hi: "<<bufheader->l_time[1] << std::endl; // microseconds since time lo
       //       char sbuf[1000]; 
       //       f_ut_utime(bufheader->l_time[0], bufheader->l_time[1], sbuf);
       //       std::cout <<"\ttimestring: " << sbuf << std::endl;
              
        //    cout <<"\tTime: " << ctime((const time_t*) (void*) &(bufheader->l_time[0]));
        //   cout << "\t\t\t + "<<bufheader->l_time[1] << " s"<<endl;
         }


    }
    
      //            std::cout <<"\tdatalen: "<<head->l_dlen << std::endl;
   //            std::cout <<"\ttime lo: "<<head->l_time[0] << std::endl; // seconds since epoch 1970
   //            std::cout <<"\ttime hi: "<<head->l_time[1] << std::endl; // microseconds since time lo
   //            char sbuf[1000]; f_ut_utime(head->l_time[0], head->l_time[1], sbuf);
   //            std::cout <<"\ttimestring: " << sbuf << std::endl;
   //            std::cout <<"\ttype: "<<head->i_type << std::endl;
   //            std::cout <<"\tsubtype: "<<head->i_subtype << std::endl;

    

   fMbsEvent = 0; // reset to avoid invalid pointer if analysis is changed in between
   fRawEvent = 0;
   fAnlEvent = 0;
   fEvents=0;
   return 0;
}

//-----------------------------------------------------------
Int_t TLdaAnalysis::UserEventFunc()
{
//// This function is called once for each event.
   Int_t value = 0;
   Int_t count = 0;
   if (fMbsEvent)
		value = fMbsEvent->GetDlen() / 2 + 2; // total longwords
	fSize->Fill(value); // fill histogram
   fEvents++;
   if(fEvents == 1 || IsNewInputFile()) {
      if(fMbsEvent) {
         count=fMbsEvent->GetCount();
         cout << "\nFirst event #: " << count  << endl;
         s_bufhe* bufheader=fMbsEvent->GetMbsBufferHeader();
         if(bufheader) {
            cout <<"First Buffer:"<<endl;
            cout <<"\tNumber: "<<bufheader->l_buf << endl;
            cout <<"\tTime: " << ctime((const time_t*) (void*) &(bufheader->l_time[0]));
            cout << "\t\t\t + "<< bufheader->l_time[1] << " s"<<endl;
            cout<<endl;         }
      }
      SetNewInputFile(kFALSE); // we have to reset the newfile flag
   }
   fLastEvent = count;
   return 0;
}
