// $Id: TLdaAnlProc.cxx 520 2009-11-10 16:15:53Z goofy $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TLdaAnlProc.h"

#include <math.h>
#include <string>

#include "TH1.h"
#include "TH2.h"
#include "Riostream.h"
#include "TROOT.h"
#include "TSystem.h"
#include <TArrayL64.h>

#include "TGo4WinCond.h"
#include "TGo4Analysis.h"

#include "TLdaAnlEvent.h"
#include "TLdaUnpackEvent.h"
#include "TLdaParameters.h"
#include "TLdaInputDescription.h"

//#include "FixedQueue.h" 

//-----------------------------------------------------------
TLdaAnlProc::TLdaAnlProc() :
		TGo4EventProcessor() {
}
//-----------------------------------------------------------
TLdaAnlProc::TLdaAnlProc(const char* name) :
		TGo4EventProcessor(name) {
	cout << "**** TLdaAnlProc: Create" << endl;

	fPar = (TLdaParameters *) GetParameter("TimeParameters");
    
    Init();
    InitSpectra();
    InitConditions();
    InitLvSpectra();
  
    

    
    
	lifeTimeClock = 0;
	lastCl = 0;
	firstRun = 0;
	mbsStartTime = 0;
	lastMbsTime = 0;
	timeSinceInj = 0;
	lastInjTime = 0;
	lastDead = 0;
	usefulLifeTime = 0;
	lastEventCount = 0;
	mbsDataSize = 0;
	lastSwitchTimeMbs = 0;
	ownEventCount = 0;
	lastLvCalculatedStep = -1;
	lastCobraWL = 0;
	lastAtosWL = 0;
	goFitting = 0;
	timeAxisValue = 0;
	ignoreInput = 0;
	ignoreInputOld = 0;
	timeLastQswInBins = 0;
	timeLastQswInBinsOld = 0;
	qSwExists = 0;
	atosAvgWL = 0;


    iLaserStep=0;
    iOldSetWaveLength = 0;
    iSetWaveLength = 0 ;
    


	for (int i = 0; i < MAX_TDC; i++) {
		tdcConvol[i].Set(((Int_t) tdc_high +1));	// ceil(fPar->period));
//        tdcConvol[i].Set(((Int_t) 30000));
	}


	


	



}
//-----------------------------------------------------------
TLdaAnlProc::~TLdaAnlProc() {
	cout << "**** TLdaAnlProc: Delete" << endl;

}
//-----------------------------------------------------------
Bool_t TLdaAnlProc::BuildEvent(TGo4EventElement * dest)  {
    
    
    
	Bool_t isValid = kFALSE; // validity of output event

	TLdaUnpackEvent* inp_evt = (TLdaUnpackEvent*) GetInputEvent();
//	TLdaAnlEvent* out_evt = (TLdaAnlEvent*) dest;
    Zero();
	// see comments in UnpackProc
	if ((inp_evt == 0) || !inp_evt->IsValid()) { // input invalid
//		out_evt->SetValid(isValid); // invalid
		return isValid; // must be same is for SetValid
	} else {
	
 //   Zero();  
        
    ownEventCount++;
    uCounter++;
	
    //Fill scaler histogram	
 //   Zero();
    

  
  	for (int i = 0; i < MAX_SCA; i++) {
		scalSE[i] = inp_evt->scalerSubEvent[i];      
        trendAv[i] += scalSE[i]; 

 //       if(i==NORTH){
 //           b1aCondMaxCounts[CH_NORTH] = wcCondMaxCounts[CH_NORTH]->Test(scalSE[i]);
//        } else if(i=MIDDLE){
//            b1aCondMaxCounts[CH_MIDDLE] = wcCondMaxCounts[CH_MIDDLE]->Test(scalSE[i]));
//        } else if (i=ANODE){
//            b1aCondMaxCounts[CH_XUV] = wcCondMaxCounts[CH_XUV]->Test(scalSE[i]));
//        } else if (i=SOUTH){
//            b1aCondMaxCounts[CH_SOUTH] = wcCondMaxCounts[CH_SOUTH]->Test(scalSE[i]));
//        } else if (i=COPPER){
//            b1aCondMaxCounts[CH_CU] = wcCondMaxCounts[CH_CU]->Test(scalSE[i]));
//        }
    }
  
        lStepTimer += scalSE[SC1MHZ];    
        lStepTimer2 += scalSE[SC1MHZ];  

  
  	//Readout TDC Data and fill TDC-trending averager
	for (int i = 0; i < MAX_TDC; i++) {
		tdcSE[i] = inp_evt->tdcSubEvent[i];
        tdcCSCounter[i]  = inp_evt->arrCSCounter[i];
/////		tsCS[i] = inp_evt->arrTimeStampCS[i];
//		for (int j = 0; j < tdcSE[i].GetSize(); j++) {
//			tdcHitCount[i]++; //purpose: trending and comparing with scaler;
//		} // Warum einfach, wenn es auch umstndlich...

        
//        tdcHitCount[i] = tdcSE[i].GetSize();
        
  //      tdcCSCounter[i] = inp_evt->arrCSCounter[i];
	}
 
Int_t iTDCsize[MAX_TDC];  
for (int i = 0; i < MAX_TDC; i++) {
    iTDCsize[i] = 0;
} 

  for (int i = 0; i < MAX_TDC; i++) {

        iTDCsize[i] = tdcSE[i].GetSize();
        spTDCsize[i]->Fill(iTDCsize[i]);
        tdcHitCount[i] = iTDCsize[i];
        
        //switch(iTDCType)
        
        if(i==NORTH){
            b1aCondMaxCounts[CH_NORTH] = wcCondMaxCounts[CH_NORTH]->Test(iTDCsize[i]);
        } else if(i==MIDDLE){
            b1aCondMaxCounts[CH_MIDDLE] = wcCondMaxCounts[CH_MIDDLE]->Test(iTDCsize[i]);
        } else if (i==ANODE){
            b1aCondMaxCounts[CH_XUV] = wcCondMaxCounts[CH_XUV]->Test(iTDCsize[i]);
        } else if (i==SOUTH){
            b1aCondMaxCounts[CH_SOUTH] = wcCondMaxCounts[CH_SOUTH]->Test(iTDCsize[i]);
        } else if (i==COPPER){
            b1aCondMaxCounts[CH_CU] = wcCondMaxCounts[CH_CU]->Test(iTDCsize[i]);
        } else if (i == NORTH_GATED) {
            b1aCondMaxCounts[CH_NORTH_GATED] = wcCondMaxCounts[CH_NORTH_GATED]->Test(iTDCsize[i]);  //-- 11:30 05-12-2022: NORTH_GATED added
        } else if (i == MIDDLE_GATED) {
            b1aCondMaxCounts[CH_MIDDLE_GATED] = wcCondMaxCounts[CH_MIDDLE_GATED]->Test(iTDCsize[i]);  //-- 11:30 05-12-2022: MIDDLE_GATED added
        }
      
// ToDo: Implement general Test on large TDC events, in particular if one or two of the buffers were full....
 //       if(iTDCsize < 8000){ 
 //               b1a_TDCValid[i]=kTRUE;
 //               b1_ValidTDC=kTRUE;
            
        }

//  b1_ValidTDC=kTRUE;
    
   if (scalSE[NEXTSTEP]){ 
       if (fPar->pusFoldStepSpectra > 0) 
       {
//           iLaserStep = iLaserStep % (fPar->piNOfLaserSteps);
           fPar->piLaserStep = fPar->piLaserStep % (fPar->piNOfLaserSteps);
       }
       //iLaserStep++;
       fPar->piLaserStep = fPar->piLaserStep +1;
       iLaserStep = fPar->piLaserStep ;
      // cout << "Next Step: " << iLaserStep <<  endl; 
       b1TestStep = kTRUE;
       cout << "if[NextStep] step: " << iLaserStep << //" from setwavelength: " << //iLaserStepFromSetWaqvelength <<
       // " " << iOldSetWaveLength <<
            "\t\t" << lStepTimer << "\t\t" << (lStepTimer-lStepTimerOld) <<endl;
            spTimeBetweenSteps->Fill((lStepTimer-lStepTimerOld)/1e6); 
       lStepTimerOld = lStepTimer;
   }

  //Alternative way to produce lasr Steps
 
  
  	if(inp_evt->eventLv[NR_LV_COBRASETWL] > 0)
    {
        iSetWaveLength  = inp_evt->eventLv[NR_LV_COBRASETWL];
        if (iSetWaveLength < iOldSetWaveLength)
        {
            iLaserStepFromSetWaqvelength++;  
            b1TestStep = kTRUE;
            cout << "a - Set Wavelength: "  << iSetWaveLength <<endl;
 //           cout << "if[Set<] step: " << iLaserStep << " from setwavelength: " << iLaserStepFromSetWaqvelength <<
//            "\t\t" << lStepTimer2 << "\t\t" << (lStepTimer2-lStepTimer2Old) <<endl;
          //  b1NewLaserStep = kTRUE;       
            lStepTimer2Old = lStepTimer2;
        } else if (iSetWaveLength > iOldSetWaveLength)
        {
            b1TestStep = kTRUE;
            iLaserStepFromSetWaqvelength=0;               
            cout << "b - Set Wavelength: "  << iSetWaveLength <<endl;         
 //           cout << "if[Set>] scan: " << iLaserStep << " from setwavelength: " << iLaserStepFromSetWaqvelength <<endl;
        //    iLaserScan++;
        //    if (iLaserScan % 2 == 0)
        //    {
         //       iLaserStepFromSetWaqvelength=0;
        //    }
        }

        iOldSetWaveLength  = inp_evt->eventLv[NR_LV_COBRASETWL];
    }


// b1_ValidTDC=kFALSE;
    

    ignoreInput = (Int_t) scalSE[LASER_BUSY_CH];
	ignoreInputOld = ignoreInput; 
    
    for (Int_t n=0; n < tdcCSCounter[QSWITCH].GetSize(); n++)
    {
        iCSClockQswitch = tdcCSCounter[QSWITCH][n];
              uiHitQswitch = tdcSE[QSWITCH][n] ;
//        cout<< "QSWITCH  CSClock \t" << iCSClockQswitch  << "\t"<< uiHitQswitch << "\t" <<  deqCSQSwitch.size() << endl;
        FillFixedDeque(deqCSQSwitch, iCSClockQswitch, deqMAX);
    }
    
    for (Int_t n=0; n < tdcCSCounter[QSWITCH500US].GetSize(); n++)
    {
        iCSClockQswitch500us = tdcCSCounter[QSWITCH500US][n];
        uiHitQswitch500us = tdcSE[QSWITCH500US][n]; 
  //      cout<< "QSWITCH 500us \t" << iCSClockQswitch500us  << "\t"<< uiHitQswitch500us << endl;
    }
    
  //  UInt_t CSClock = tdcCSCounter[i][j];
    
     /*               if(trendLvCt[NR_LV_COBRASETWL]>0)
                {
     //               pmtOverWavelength->Fill(iSetWaveLength / 1e6TWL] * 1000000.), pmtSignal);
		// this is the same as in the loop below
                    
                    for (int i = 0; i < MAX_TDC; i++) {
                        PMToverWavelengHist[i]->Fill(iSetWaveLength / 1e6, trendAv[i]);
                    }
                }
    */
    
    for (int i = 0; i < MAX_TDC; i++) {
        for (int j = 0; j < tdcSE[i].GetSize(); j++) {
            UInt_t hit = tdcSE[i][j];
            hit = hit + fPar->piPhaseOffset;            
            Int_t ihit;
            Long64_t CSClock = tdcCSCounter[i][j];
            Bool_t b1InCSWindow =kFALSE;
            Int_t iTDCType;
//            cout << "\t" << j <<"\t" <<  "CSCLOCK " << CSClock <<endl;
            iTDCType = i;
            ihit = (Int_t) floor(fmod(hit, fPeriodInBins));
            
            stackedTdcHist[i]->Fill(ihit);
            
            accumulatedTdcHist[i]->Fill(ihit);
            

       //     PMToverWavelengHist[i]->Fill(iSetWaveLength / 1e6);
            
            // Remainder of devision, then to next lower integer...
            
         //testcode //      if(i==QSWITCH){cout<< j << "\t" << "QSWITCh  CSClock \t" << CSClock  << endl;
        //        iQswitchCountTemp = CSClock;
                 
          //  }
          //  if(i==QSWITCH500US){
         //           iQswitchCountDiff = iQswitchCountTemp - CSClock;
        //        cout <<"\t"<< "QSWITCh 500us CSClock \t" << CSClock  << " Diff: "  << iQswitchCountDiff << endl;}
            if(ignoreInput == 0)
            {
                spStepSig[CH_N]->Fill(iLaserStep);
            }
            
            switch(iTDCType)
            {
                case MIDDLE:
                    b1aCondTDCSignal[CH_MIDDLE]=wcCondTDCSignal[CH_MIDDLE]->Test(ihit);
                    
                    for(UInt_t n=0;  n<deqCSQSwitch.size(); n++)
                    {
                        if(((CSClock-deqCSQSwitch[n]) < fPar->piNOfCSForValidTimeRange) && (CSClock-deqCSQSwitch[n] >0))
                        {
                            b1InCSWindow =kTRUE;
 //                           cout << "Hit in CS Window:\t" << CSClock << "\t" << hit << "\t" << n << "\t" << j << endl;
                        }
                    }
                    
                    if(b1InCSWindow && (ignoreInput == 0))
                    {
                        accusigTdcHist[MIDDLE]->Fill(ihit);
                        
                        if((iLaserStep > iLaserStepLow) && (iLaserStep < iLaserStepHigh))
                        {
                            accusigstepTdcHist[MIDDLE]->Fill(ihit);
                        }
                    }
                    
                    if(b1InCSWindow  && 
                        b1aCondTDCSignal[CH_MIDDLE] &&         b1aCondMaxCounts[CH_MIDDLE] &&
                        (ignoreInput == 0))
                    {            
                        spStepSig[CH_MIDDLE]->Fill(iLaserStep);
                        spStepSig[CH_ALL]->Fill(iLaserStep);  
                        PMToverWavelengHist[MIDDLE]->Fill(iSetWaveLength / 1e6);                                            
                    } ;
                    
                    break;                
                case MIDDLE_GATED:
                    b1aCondTDCSignal[CH_MIDDLE_GATED]=wcCondTDCSignal[CH_MIDDLE_GATED]->Test(ihit);
                    

                        
                    if(b1aCondTDCSignal[CH_MIDDLE_GATED] &&       b1aCondMaxCounts[CH_MIDDLE_GATED] &&
                        (ignoreInput == 0))
                    {            
                        spStepSig[CH_MIDDLE_GATED]->Fill(iLaserStep);    accusigstepTdcHist[MIDDLE_GATED]->Fill(ihit);                PMToverWavelengHist[MIDDLE_GATED]->Fill(iSetWaveLength / 1e6);    
                    } ;
                    
                    break;
                case NORTH:
                    b1aCondTDCSignal[CH_NORTH]=wcCondTDCSignal[CH_NORTH]->Test(ihit);
                    
                    for(UInt_t n=0;  n<deqCSQSwitch.size(); n++)
                    {
                        if((CSClock-deqCSQSwitch[n] < fPar->piNOfCSForValidTimeRange) && (CSClock-deqCSQSwitch[n] >0))
                        {
                            b1InCSWindow =kTRUE;
   //                         cout << "Hit in CS Window:\t" << CSClock << "\t" << hit << "\t" << n << "\t" << j << endl;
                        }
                    }
                    if(b1InCSWindow &&(ignoreInput == 0))
                    {
                        accusigTdcHist[NORTH]->Fill(ihit);
                        if((iLaserStep > iLaserStepLow) && (iLaserStep < iLaserStepHigh))
                        {
                            accusigstepTdcHist[NORTH]->Fill(ihit);
                        }
                    };
                    
                    if(b1InCSWindow  && 
                        b1aCondTDCSignal[CH_NORTH] &&         b1aCondMaxCounts[CH_NORTH] &&
                        (ignoreInput == 0))
                    {            
                        spStepSig[CH_NORTH]->Fill(iLaserStep); //tdcSE[NORTH].GetSize());          
                        spStepSig[CH_ALL]->Fill(iLaserStep); //tdcSE[NORTH].GetSize());
                        PMToverWavelengHist[NORTH]->Fill(iSetWaveLength / 1e6); 
                    } ;
                    break;
                case NORTH_GATED:
                    b1aCondTDCSignal[CH_NORTH_GATED]=wcCondTDCSignal[CH_NORTH_GATED]->Test(ihit);
                    
                    //Since this is a hardware Window after QSwitch, the software windows is not need 
//                    for(UInt_t n=0;  n<deqCSQSwitch.size(); n++)
//                    {
//                        if((CSClock-deqCSQSwitch[n] < fPar->piNOfCSForValidTimeRange) && (CSClock-deqCSQSwitch[n] >0))
 //                       {
 //                           b1InCSWindow =kTRUE;
 ////                           cout << "Hit in CS Window:\t" << CSClock << "\t" << hit << "\t" << n << "\t" << j << endl;
                    //    }
                   // }
                  //  if(b1InCSWindow  && 
                        
                    if(b1aCondTDCSignal[CH_NORTH_GATED] &&         b1aCondMaxCounts[CH_NORTH_GATED] &&
                        (ignoreInput == 0))
                    {            
                        spStepSig[CH_NORTH_GATED]->Fill(iLaserStep); PMToverWavelengHist[NORTH_GATED]->Fill(iSetWaveLength / 1e6);  accusigstepTdcHist[NORTH_GATED]->Fill(ihit);
                    } ;
                    
                    break;          
                case SOUTH:
                    b1aCondTDCSignal[CH_SOUTH]=wcCondTDCSignal[CH_SOUTH]->Test(ihit);
                    
                    for(UInt_t n=0;  n<deqCSQSwitch.size(); n++)
                    {
                        if((CSClock-deqCSQSwitch[n] < fPar->piNOfCSForValidTimeRange) && (CSClock-deqCSQSwitch[n] >0))
                        {
                            b1InCSWindow =kTRUE;
   //                         cout << "Hit in CS Window:\t" << CSClock << "\t" << hit << "\t" << n << "\t" << j << endl;
                        }
                    }
                    if(b1InCSWindow  && 
                        b1aCondTDCSignal[CH_SOUTH] &&         b1aCondMaxCounts[CH_SOUTH] &&
                        (ignoreInput == 0))
                    {            
                        spStepSig[CH_SOUTH]->Fill(iLaserStep); //tdcSE[NORTH].GetSize());          
                        PMToverWavelengHist[SOUTH]->Fill(iSetWaveLength / 1e6);  accusigstepTdcHist[SOUTH]->Fill(ihit);   
                    } ;
                    break;                                       
                case ANODE:
                    b1aCondTDCSignal[CH_XUV]=wcCondTDCSignal[CH_XUV]->Test(ihit);
                    
                    for(UInt_t n=0;  n<deqCSQSwitch.size(); n++)
                    {
                        if((CSClock-deqCSQSwitch[n] < fPar->piNOfCSForValidTimeRange) && (CSClock-deqCSQSwitch[n] >0))
                        {
                            b1InCSWindow =kTRUE;
//                            cout << "Hit in CS Window:\t" << CSClock << "\t" << hit << "\t" << n << "\t" << j << endl;
                        }
                    }
                    
                    
                    
                    if(b1InCSWindow  && 
                        b1aCondTDCSignal[CH_XUV] && 
                        b1aCondMaxCounts[CH_XUV] &&
                        (ignoreInput == 0))
                    {            
                        spStepSig[CH_XUV]->Fill(iLaserStep); //tdcSE[NORTH].GetSize());          
                        PMToverWavelengHist[ANODE]->Fill(iSetWaveLength / 1e6); accusigstepTdcHist[ANODE]->Fill(ihit);                       
                    } ;                  
                    
                    break;
                case COPPER:                    
                    b1aCondTDCSignal[CH_CU]=wcCondTDCSignal[CH_CU]->Test(ihit); PMToverWavelengHist[COPPER]->Fill(iSetWaveLength / 1e6); 
                case PDSO:
                    PMToverWavelengHist[PDSO]->Fill(iSetWaveLength / 1e6); 
                    break;
                case PDNO:
                    PMToverWavelengHist[PDNO]->Fill(iSetWaveLength / 1e6); 
                    break;
                case PART_SA:
                    PMToverWavelengHist[PART_SA]->Fill(iSetWaveLength / 1e6); 
                    break;   
                case PART_NA:
                    PMToverWavelengHist[PART_NA]->Fill(iSetWaveLength / 1e6); 
                    break;  
                default: 
                    PMToverWavelengHist[i]->Fill(iSetWaveLength / 1e6); 
                    break;
                    
            }


        
            
            if (ignoreInput == 0) 
            { //Laser not busy, accumulate and show!
                tdcConvol[i][ihit]++;
            }        
        } 

		//}
    } 
    
    
    if(ignoreInput == 0){             
        
         
    }
    
    spStep[CH_N]->Fill(iLaserStep);
    spStep[CH_1MHZ]->Fill(iLaserStep, scalSE[SC1MHZ]);
    spStep[CH_NORTH]->Fill(iLaserStep, tdcSE[NORTH].GetSize());
    spStep[CH_MIDDLE]->Fill(iLaserStep, tdcSE[MIDDLE].GetSize());
    spStep[CH_ALL]->Fill(iLaserStep, tdcSE[MIDDLE].GetSize()); 
    spStep[CH_ALL]->Fill(iLaserStep, tdcSE[NORTH].GetSize()); 
    spStep[CH_XUV]->Fill(iLaserStep,tdcSE[ANODE].GetSize());     
    spStep[CH_CU]->Fill(iLaserStep, tdcSE[COPPER].GetSize());    
    spStep[CH_SA_SZ]->Fill(iLaserStep, tdcSE[PART_SA].GetSize());     spStep[CH_NA_SZ]->Fill(iLaserStep, tdcSE[PART_NA].GetSize());     
    spStep[CH_MIDDLE_GATED]->Fill(iLaserStep, tdcSE[MIDDLE_GATED].GetSize()); 
    spStep[CH_NORTH_GATED]->Fill(iLaserStep, tdcSE[NORTH_GATED].GetSize()); 

    
//    if(uCounter > 1000)
//    {
//        NormalizeHistorgams();  
//        uCounter = 0;
//    }    
    
 

	//readout scaler values and fill trending averager
	lifeTimeClock += ((Double_t) scalSE[CLOCK_CH]) / (Double_t) CLOCK_FREQ;
  


	for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
		if (inp_evt->eventLv[i] > 0) {
//			trendLvAv[i] = inp_evt->eventLv[i]; // just take last value from Labview JUW!
			trendLvAv[i] += inp_evt->eventLv[i]; // first, better method. the other one was just a workaround, because of the UInt_t caused overflow bugs.
			trendLvCt[i]++;

		}
	}
	


	
	if ((inp_evt->eventLv[NR_LV_COBRASETWL] > 0) && (COBRA_RAW_TO_NM >0)  ) {
		lastCobraWL = (Double_t)(
				inp_evt->eventLv[NR_LV_COBRASETWL]) / (Double_t)COBRA_RAW_TO_NM;
		//this is a backup. One should rather use the averaged value in the next Go4 steps
    }
	if ((inp_evt->eventLv[NR_LV_ATOSWL] > 0) && (ATOS_RAW_TO_NM > 0)) {
		lastAtosWL = (Double_t)(
				inp_evt->eventLv[NR_LV_ATOSWL]) / (Double_t)ATOS_RAW_TO_NM;}
    
		//Status=4 bedeutet "waiting", also dwell time
	if (inp_evt->eventLv[NR_LV_COBSTAT] == 4) { //jetzt glaube alles, auch stepNr=0. frueher so was: inp_evt->eventLv[NR_LV_COBSTEPNR] > 0
		lastLvCalculatedStep = inp_evt->eventLv[NR_LV_COBSTEPNR];
	}   ///?????????

	mbsDataSize += inp_evt->eventSizeMbs;

	//Fill "time of event histogram"
//	if (fPar2->flagHistStepAnlDiagnosis != 0) {
    
    
    timeOfEventSec->SetBinContent(ownEventCount,
				(Double_t) inp_evt->eventTime);
    timeOfEventmSec->SetBinContent(ownEventCount,
				inp_evt->eventTimeMilliSec);
		//		cout<< "event sec"<<(Double_t)inp_evt->eventTime;
		//		cout<< "MBSevent ct"<<(Double_t)inp_evt->eventCount;
//	}
    

	mbsTime = (Double_t) inp_evt->eventTime
			+ (Double_t) inp_evt->eventTimeMilliSec / 1000;
	if (firstRun == 0) {//	 Get MBStime of first event in sec and calculate elapsed time since start
		mbsStartTime = mbsTime;
		timeSinceInj = 0;
		lastInjTime = 0;
		firstRun = 1;
	}
//	if (scalSE[INJECTION_CH] != 0) {
//		lastInjTime = mbsTime;
//	}
	mbsTimeElapsed = mbsTime - mbsStartTime;
//	timeSinceInj = mbsTime - lastInjTime;

	//Second gone? Now in MBS-time!
    
	if (mbsTimeElapsed - floor(lastMbsTime) > 1) {
		for (timeAxisValue = (Int_t) ceil(lastMbsTime) % 7200;//before TREND_SCAL_HIGH
				timeAxisValue <= (Int_t) floor(mbsTimeElapsed) % 7200;//before TREND_SCAL_HIGH
				timeAxisValue++) {

			Double_t lifeTimeDAQ = lifeTimeClock - lastCl;// only used for trending, not for averaging during WL-step!
			// Fill Scaler Trending
        if (lifeTimeDAQ > 0)
        {
				for (int i = 0; i < MAX_SCA; i++) {
					scalerTrending[i]->SetBinContent(timeAxisValue,
							trendAv[i] / lifeTimeDAQ);//set bin to average of last 1 seconds
				}
				for (int i = 0; i < MAX_TDC; i++) {
					tdcTrending[i]->SetBinContent(timeAxisValue,
							tdcHitCount[i] / lifeTimeDAQ);//set bin to average of last second
				}				
        }
				dcTrafoTrending->SetBinContent(timeAxisValue,
						((trendAv[DCTRAFO_CH] / lifeTimeDAQ) - DCTRAFO_OFFSET)
								/ DCTRAFO_PROP);//set bin to average of last 1 seconds



				// Labview data, only values that change need avering such as voltage, but not the set values				
                
				for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
					if (trendLvCt[i] != 0) {
						lvTrending[i]->SetBinContent(timeAxisValue,
//								trendLvAv[i]); // set bin content to last value received in last second JUW!
								trendLvAv[i] / trendLvCt[i]);//set bin to average of last second
					};
				}
				
				
				if (trendLvCt[NR_LV_VOLTECOOL] > 0) {
					Double_t agilentMeas = (trendLvAv[NR_LV_VOLTECOOL]
							/ trendLvCt[NR_LV_VOLTECOOL]) / AGILENT_PROP;
					agilentTrend->SetBinContent(timeAxisValue, agilentMeas);

					Double_t uSetAvg;	
                    Double_t ohmlabsMeas = agilentMeas * AGILENT_RATIO;
                    
                    if(lifeTimeDAQ > 0)
                    {
                        uSetAvg = (trendAv[USET_CH] / lifeTimeDAQ
							- USET_OFFSET) / USET_PROP;
                            //??????????
                        uSetTrend->SetBinContent(timeAxisValue, uSetAvg);					deviationTrendRead->SetBinContent(timeAxisValue,
							uSetAvg - ohmlabsMeas);
                    }

				
					ohmLabsTrend->SetBinContent(timeAxisValue, ohmlabsMeas);

					deviationTrendParam->SetBinContent(timeAxisValue,fPar->pfUcool - ohmlabsMeas);
				}
				
				
				
				//################################################## Fill PMT over wavelength histogramm #################################################
				// test to fill pmt over wavelength 4.4.2022
				//Double_t laserWavelength = trendLvAv[NR_LV_ATOSWL] / (trendLvCt[NR_LV_ATOSWL] * 1000000.);				
			//	Double_t pmtSignal = trendAv[5];			
				
 /*               if(trendLvCt[NR_LV_COBRASETWL]>0)
                {
     //               pmtOverWavelength->Fill(trendLvAv[NR_LV_COBRASETWL] / (trendLvCt[NR_LV_COBRASETWL] * 1000000.), pmtSignal);
		// this is the same as in the loop below
                    
                    for (int i = 0; i < MAX_TDC; i++) {
                        PMToverWavelengHist[i]->Fill(trendLvAv[NR_LV_COBRASETWL] / (trendLvCt[NR_LV_COBRASETWL] * 1000000.), trendAv[i]);
                    }
                }
*/
   //             if (lifeTimeDAQ > 0){
	//			missingTdcHits->SetBinContent(timeAxisValue,
	//					(trendAv[MAINPMT_CH] - tdcHitCount[MAINPMT_CH])
	//							/ lifeTimeDAQ);
      //          }
                
				// Fill size of event trending
				if (inp_evt->eventCount > lastEventCount){
                sizeOfEvent->SetBinContent(timeAxisValue,
						mbsDataSize / (inp_evt->eventCount - lastEventCount));
                durationTrend->SetBinContent(timeAxisValue,
						1e3 * lifeTimeDAQ
								/ (inp_evt->eventCount - lastEventCount));
                }
				//	Fill Dead Time trending
				deadTimeIntegral->SetBinContent(timeAxisValue,
						mbsTimeElapsed - lifeTimeClock);
				deadTimeNow->SetBinContent(timeAxisValue,
						(mbsTimeElapsed - lifeTimeClock) - lastDead);
				
		}  
		// reset remember-values
		mbsDataSize = 0;
		lastDead = mbsTimeElapsed - lifeTimeClock;
		lastMbsTime = mbsTimeElapsed;
		lastCl = lifeTimeClock;
		lastEventCount = inp_evt->eventCount;
        
		for (int i = 0; i < MAX_SCA; i++) {
			trendAv[i] = 0;
		}
		for (int i = 0; i < MAX_TDC; i++) {
			tdcHitCount[i] = 0;
		}

		for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
			trendLvAv[i] = 0;
			trendLvCt[i] = 0;
		}

	}


		
		
		// ############ integral for signal - reference bunch ############
		//if(trendAv[31]>0.5 && trendAv[31]<1.5){		
        if(trendAv[30]>0.5 && trendAv[30]<1.5){
			for (int i = 0; i < MAX_TDC; i++) {
                stackedTdcHist[i]->Reset();
			}
//			cout<<IntegralValSigMinRef[3]<<endl;
		}
		
		
		
//		if (qSwExists > 0) {
//			qSwitchInterval->Fill(
//					(timeLastQswInBins - timeLastQswInBinsOld)
//							/ FREQ_RATIO_TDC_TO_TSCS / 1000); //Fill Q-switch distance Hist
//			timeLastQswInBinsOld = timeLastQswInBins;
//
//		}

		//Fill averager for this wavelength step
		for (int i = 0; i < MAX_SCA; i++) {
			lasingAv[i] += scalSE[i];
		}
		for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
			if (inp_evt->eventLv[i] > 0) {
				lasingLvAv[i] += (ULong64_t) inp_evt->eventLv[i];
				lasingLvCt[i]++;
//				if (i==NR_LV_ATOSWL){\\only debugging use
//					cout<<"lasingLvAv[NR_LV_ATOSWL]="<<lasingLvAv[NR_LV_ATOSWL]<<"\t lasingLvCt[NR_LV_ATOSWL]="<<lasingLvCt[NR_LV_ATOSWL]<<endl;
//				}

			}
		}

		usefulLifeTime += ((Double_t) scalSE[CLOCK_CH]) / (Double_t) CLOCK_FREQ;


    //out_evt->SetValid(isValid);
	return isValid;
// see comments in UnpackProc
//
}}

////////////////////////////////////////////

void TLdaAnlProc::InitSpectra()
{
    for (Int_t i = 0; i < MAX_SCA; i++) {
		sprintf(tmpPath, "1_Trending/Scaler/TrS%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TrS%i_%s", i, channelNames[i].c_str());
		scalerTrending[i] = MakeTH1('D', tmpPath, tmpTitle, trend_scal_div,
				trend_scal_low, trend_scal_high, "MBS time [s]",
				"cps (dead time corrected)");
	}

	dcTrafoTrending = MakeTH1('D', "1_Trending/TrDCTrafo", "DCTrafo[mA]",
			trend_scal_div, trend_scal_low, trend_scal_high, "MBS time [s]",
			"Ion current [mA]");

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "1_Trending/TDC/TrT%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TrT%i_%s", i ,channelNames[i].c_str());
		tdcTrending[i] = MakeTH1('D', tmpPath, tmpTitle, trend_scal_div,
				trend_scal_low, trend_scal_high, "MBS time [s]",
				"cps (only scaler-dead-time corrected)");
	}


        
	timeOfEventSec = MakeTH1('D', "2_Diag/Time_of_Event_sec",
			"Time_of_Event_sec", time_div, time_low, time_high, "event no",
			"time [s]");

	timeOfEventmSec = MakeTH1('D', "2_Diag/Time_of_Event_msec",
			"Time_of_Event_msec", time_div, time_low, time_high, "event no",
			"time [ms]");

	sizeOfEvent = MakeTH1('D', "2_Diag/Size_of_MBS_Event",
			"Size_of_MBS_Event", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]", "average eventsize this sec [long words]");

	deadTimeIntegral = MakeTH1('D', "2_Diag/ScDead_Time_integral",
			"Dead_Time_integral", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]", "accumulated scaler dead time");

	deadTimeNow = MakeTH1('D', "2_Diag/ScDead_Time_every_sec",
			"Dead_Time_every_sec", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]", "scaler dead time in this sec [s]");

	missingTdcHits = MakeTH1('D', "2_Diag/Missing_TDC_hits",
			"Missing_TDC_hits", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]", "Missing_TDC_hits in this sec [s]");

	durationTrend = MakeTH1('D', "2_Diag/Event_Duration",
			"Event Lifetime", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]", "Avg Event lifetime this sec [ms]");
    
    
	agilentTrend = MakeTH1('D', "2_Diag/AgilentVoltage",
			"Agilent Voltage", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]",
			"Avg measured Agilent Voltage this sec [V]");

	uSetTrend = MakeTH1('D', "2_Diag/Heinzinger_SetVoltage",
			"Set Voltage", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]", "Electron Cooler Set Voltage this sec [V]");

	ohmLabsTrend = MakeTH1('D', "2_Diag/PTB_HVDC_Voltage",
			"Ohmlabs Voltage", trend_scal_div, 1, trend_scal_high,
			"elapsed MBS time [s]",
			"Avg measured OhmLabs Voltage this sec [V]");

	deviationTrendRead = MakeTH1('D',
			"2_Diag/VoltageDeviationRead",
			"Voltage Deviation read and measured", trend_scal_div, 1,
			trend_scal_high, "elapsed MBS time [s]",
			"read Uset - Ohmlabs measured Voltage [V]");

	deviationTrendParam = MakeTH1('D',
			"2_Diag/VoltageDeviationParam",
			"Voltage Deviation parameter and measured", trend_scal_div, 1,
			trend_scal_high, "elapsed MBS time [s]",
			"Uset from parmeter - Ohmlabs measured Voltage [V]");
    
const Double_t scal_low = -0.0005;
const Double_t scal_high = 20.0005;
const Int_t scal_div = 20001;
    
    spTimeBetweenSteps = MakeTH1('I',
			"2_Diag/TimeBetweenSteps",
			"Time between steps using scaler", scal_div, scal_low,
			scal_high, "N",
			"time");
    spTimeBetweenSteps2 = MakeTH1('I',
			"2_Diag/TimeBetweenSteps2",
			"Time between steps using Set Wavelength", scal_div, scal_low,
			scal_high, "N",
			"time");
    //spTimeBetweenSteps2;



	
	// ######### definition of TDC stacked histogram including signal and refence bunch conditions ##########
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "3_TDC/stacked/TDC%i_stacked_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TDC_stacked_%s", channelNames[i].c_str());
		stackedTdcHist[i] = MakeTH1('I', tmpPath, tmpTitle, tdc_div, tdc_low,
				tdc_high,
				"time till common stop modulo revolution period [bin]", "hits");
        sprintf(tmpName, "TDC%i_stacked_%s", i,	channelNames[i].c_str());				
		sprintf(tmpTitle, "TDC_stacked_Sig_%s", channelNames[i].c_str());
    }
    

	
	// ####### accumulated TDC histograms
		for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "3_TDC/accu/TDC%i_accumulated_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TDC_accumulated_%s", channelNames[i].c_str());
		accumulatedTdcHist[i] = MakeTH1('I', tmpPath, tmpTitle, tdc_div, tdc_low,
				tdc_high,
				"time till common stop modulo revolution period [bin]", "hits");
        }
        
        for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "3_TDC/accusig/TDC%i_accusig_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TDC_accusig_%s", channelNames[i].c_str());
		accusigTdcHist[i] = MakeTH1('I', tmpPath, tmpTitle, tdc_div, tdc_low,
				tdc_high,
				"time till common stop modulo revolution period [bin], soft CS window", "hits");
        }
	        
        
        for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "3_TDC/accusigstep/TDC%i_accusigstep_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TDC_accusigstep_%s", channelNames[i].c_str());
		accusigstepTdcHist[i] = MakeTH1('I', tmpPath, tmpTitle, tdc_div, tdc_low,
				tdc_high,
				"time till common stop modulo revolution period [bin], soft CS window, gate on laser step", "hits");
        }
	
			for (Int_t i = 0; i < MAX_TDC; i++) {
       
        sprintf(tmpPath, "3_TDC/size/TDC%i_size_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TDC_size_%s", channelNames[i].c_str());
        
        spTDCsize[i] = MakeTH1('I', tmpPath, tmpTitle, 5001, -0.5, 5000.5,
				"TDC Hit size", "hits");;
	}
	
	
	// ####### Histogram for Signal minus reference Bunch#######
//	for (Int_t i = 0; i < MAX_TDC; i++) {
//		sprintf(tmpPath, "Step2_Anl/SigMinusRef/TDC%i_SigMinusRef_%s", i,
//				channelNames[i].c_str());
//		sprintf(tmpTitle, "TDC_SigMinusRef_%s", channelNames[i].c_str());
//		SigMinusRefHist[i] = MakeTH1('D', tmpPath, tmpTitle, wavelength_div, wavelength_low,
//				wavelength_high,
//				"wavelength [nm]", "Signal - Reference Bunch [cts]");
//        
//	}
	
 
    for (Int_t i = 0; i < MAX_STEP; i++) 
    {
        sprintf(tmpPath, "4_LaserSteps/Counts/LS%i_%s", i, stepNames[i].c_str());
        sprintf(tmpTitle, "Steps%i_%s", i,
        stepNames[i].c_str());
        
        spStep[i] = MakeTH1('D', tmpPath, tmpTitle, uiStepsDiv, uiStepsMin, uiStepsMax,
        "Laser Step", "Counts");
        
//        sprintf(tmpPath, "4_LaserSteps/CountsNorm/LSN%i_%s", i, stepNames[i].c_str());
//        sprintf(tmpTitle, "StepsN%i_%s", i,
//        stepNames[i].c_str());
//        
//        spStepN[i] = MakeTH1('D', tmpPath, tmpTitle, uiStepsDiv, uiStepsMin, uiStepsMax,
//        "Laser Step", "Counts Norm N");
        
        ///////////////////////////////
        
        sprintf(tmpPath, "4_LaserSteps/Signal/LSS%i_%s", i, stepNames[i].c_str());
        sprintf(tmpTitle, "StepsS%i_%s", i,
        stepNames[i].c_str());
        
        spStepSig[i] = MakeTH1('D', tmpPath, tmpTitle, uiStepsDiv, uiStepsMin, uiStepsMax,
        "Laser Step", "Counts Signal Window");
        

            
        ////////////////////////
         
//       sprintf(tmpPath, "4_LaserSteps/SignalNorm/LSSN%i_%s", i, stepNames[i].c_str());
//       sprintf(tmpTitle, "StepsSN%i_%s", i,
//        stepNames[i].c_str());
//        
//        spStepSigN[i] = MakeTH1('D', tmpPath, tmpTitle, uiStepsDiv, uiStepsMin, uiStepsMax,
//        "Laser Step", "Counts Signal Window Norm");
       
        sprintf(tmpTitle, "Cts/WinConCts_%s",
        stepNames[i].c_str());
        wcCondMaxCounts[i] = MakeWinCond(tmpTitle, -1000,1000);	
        
        sprintf(tmpTitle, "Sig/WinConSig_%s",
        stepNames[i].c_str());
        wcCondTDCSignal[i]= MakeWinCond(tmpTitle, -1000,1000);	
        
    }
    
    
//         spStepNNotInQSwitchWindow = MakeTH1('D', "4_LaserSteps/Signal/NOutOfWIndow", "Hits Not in QSwitchWindow", uiStepsDiv, uiStepsMin, uiStepsMax, "Laser Step", "Counts Signal Window");
        
     //   spStep1MHzNotInQSwitchWindow = MakeTH1('D', "4_LaserSteps/Signal/MHzOutOfWIndow", "Hits Not in QSwitchWindow", uiStepsDiv, uiStepsMin, uiStepsMax, "Laser Step", "Counts Signal Window");

 //   qSwitchInterval = MakeTH1('D', "Step2_Anl/Lifetime/Q_Switch_interval",
//			"QSwitch_intervals", 2501, -500.5, 2000.5,
//			"time duration between two Q-switches [ms]", "Occurrences");



	
	//############################## Define Histogramm PMT over wavelength ######################################
	// create pmt over wavelength histogramm 4.4.2022
//	pmtOverWavelength = MakeTH1('D', "Step2_Anl/LabView/laserscan", "PMT over laserscan",
//		wavelength_div, wavelength_low, wavelength_high, "wavelength [nm]",
//		"PMT [cts]");
// not need same as channel numer 5 (north) below)

Double_t fHalfChannelWidth;

// Calculate the PMTOverWL spectra from SetParameters.C. If not Input : Use Default Spectrum.
if ((std::abs(fPar->pfSetWavelengthLow - fPar->pfSetWavelengthHigh) > 1e-5) && (fPar->piNOfLaserSteps >1) )
{
    fHalfChannelWidth = 0.5 * std::abs(fPar->pfSetWavelengthLow - fPar->pfSetWavelengthHigh) / (Double_t)fPar->piNOfLaserSteps; 
    if(fPar->pfSetWavelengthHigh > fPar->pfSetWavelengthLow)
    {
        wavelength_high = fPar->pfSetWavelengthHigh + fHalfChannelWidth;
        wavelength_low = fPar->pfSetWavelengthLow - fHalfChannelWidth;
    } else // In case someone mixes up the correct order in the SetParamters.C file
    {
        wavelength_high = fPar->pfSetWavelengthLow + fHalfChannelWidth;
        wavelength_low = fPar->pfSetWavelengthHigh - fHalfChannelWidth;
    }
    wavelength_div = fPar->piNOfLaserSteps + 1;
} 

	for (Int_t i = 0; i < MAX_TDC; i++) {
	sprintf(tmpPath, "5_Laserscan/TDC%i_PMToverWave_%s", i,
			channelNames[i].c_str());
	sprintf(tmpTitle, "TDC_PMToverWave_%s", channelNames[i].c_str());
	PMToverWavelengHist[i] = MakeTH1('D', tmpPath, tmpTitle, wavelength_div, wavelength_low,
			wavelength_high,
			"wavelength [nm]", "PMT [cts]");
		}
				
	//###########################################################################################################	
	


    
    ;    
};

////////////////////////////////////////////

void TLdaAnlProc::InitConditions()
{
    
  	
	//    TestConditions = MakeWinCond("TestName",0 ,10) ;
        
     if (!gSystem->AccessPathName("SetConditions.C"))
   gROOT->ProcessLine(".x SetConditions.C");
   else
     cout << "**** Cannot find SetConditions.C script in current directory ! ****" << endl;       
        
	

    ;    
};

////////////////////////////////////////////

void TLdaAnlProc::InitLvSpectra()
{
    for (Int_t i = 0; i < TOTAL_LV_NUMBERS; i++) {
		sprintf(tmpPath, "1_Trending/LV/TrLV%i_%s", i,
				lvNames[i].c_str());
		sprintf(tmpTitle, "TrLV%i_%s", i, lvNames[i].c_str());
		sprintf(tmpAxis, "LVaverage %s", lvUnits[i].c_str());
		lvTrending[i] = MakeTH1('D', tmpPath, tmpTitle, trend_scal_div, 1,
				trend_scal_high, "elapsed MBS time [s]", tmpAxis);
	}
};

////////////////////////////////////////////

void TLdaAnlProc::NormalizeHistorgams()
{
   //     for (Int_t i=0; i < MAX_STEP;i++)
   //     {
   //         spStepN[i]->Divide(spStep[i],spStep[CH_N]);
   //         spStepSigN[i]->Divide(spStepSig[i],spStepSig[CH_N]);
   //         
   //     }
};

////////////////////////////////////////////

void TLdaAnlProc::Zero()
{
//   for (int i = 0; i < MAX_SCA; i++) {
//       b1a_scalValid[i]=kFALSE;
//  }

 //   for (int i = 0; i < MAX_TDC; i++) {
 //       b1a_TDCValid[i]=kFALSE;
 //       b1a_InTDCCondWindow[i]=kFALSE;
 //   }   
    
//    for (Int_t i = 0; i < MAX_TDC; i++) {
//        b1a_TDCValid[i]=kFALSE;
//        b1a_InTDCCondWindow[i]=kFALSE;
//    }
    
      	for (int i = 0; i < MAX_SCA; i++) {
            scalSE[i] = 0;
        }
    
    for (Int_t i = 0; i < MAX_STEP; i++) {
        b1aCondMaxCounts[i]=kFALSE;
        b1aCondTDCSignal[i]=kFALSE;
    }      
    b1TestStep=kTRUE;
    fPeriodInBins = 3.0e8 / fPar->pfRevolutionFrequency;
};

////////////////////////////////////////////

void TLdaAnlProc::Init(){
    
  //  b1InQSWitchWindow = kFALSE;   
    iCSClockQswitch=0;    
    iCSClockQswitch500us=0;    
    iCSClockQswitch1ms=0;    
    iCSClockQswitchPDSO=0;    
    uiHitQswitch=0;
    uiHitQswitch500us=0;
    uiHitQswitch1ms=0;   
    uiHitQswitchPDSO=0;  
//	}
	for (int i = 0; i < MAX_SCA; i++) {
		lasingAv[i] = 0;
	}
	for (int i = 0; i < MAX_TDC; i++) {
		tdcHitCount[i] = 0;
	}
	for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
		trendLvAv[i] = 0;
		trendLvCt[i] = 0;
		lasingLvAv[i] = 0;
		lasingLvCt[i] = 0;
	}
    ;    
};

void TLdaAnlProc::FillFixedDeque(std::deque< Long64_t >& myDeque, Long64_t value, const size_t max_size)
{
    if(myDeque.size()==max_size){myDeque.pop_front();};
    myDeque.push_back(value);
};
