/*
 * TLdaInputDescription.h
 *
 *  Created on: 01.02.2011
 *      Author: mnothhel
 */

#ifndef TLDAINPUTDESCRIPTION_H_
#define TLDAINPUTDESCRIPTION_H_

#include "TGo4Analysis.h"
#include <stdlib.h>
#include <string>

#define MAX_MEM 3200000 // 12.8 Mbytes, set down by a factor of 10 compared to Shizu
#define MAX_TDC 16
#define MAX_SCA 32
#define CLOCK_CH 23
#define MAINPMT_CH 0
#define CU_GATED1_CH 7
#define CU_GATED3_CH 8
#define CU_GATED2_CH 9
#define SHUTTER_CL_TIMING_CH 12
#define Q_SWITCH_CH 15
#define DCTRAFO_CH 16
#define INJECTION_CH 17
#define USET_CH 18
#define DIVIDED_RF_CH 22

#define EXT_RF_DIVISOR 4
#define CLOCK_FREQ 1e6//(Int_t)1e5 macht Probleme
#define THIRTY_BITS 1073741824
#define FREQ_RATIO_TDC_TO_TSCS 300//the clock for TDC is 300 MHZ; for the common stop timestamp it is 1 MHz
#define REAL_IMPORTANT_FREQUENCY_TO_GET_THE_CORRECT_VALUE_VERY_SAVE 4040000

//#define LASER_SCAN_START 26
#define LASER_BUSY_CH 27
//#define LASER_SWITCH 28//next wavelength bin

#define N_BUNCH 1
#define DCTRAFO_OFFSET 2059.3//Frequenz=DCTRAFO[mA]*DCTRAFO_PROP + DCTRAFO_OFFSET
#define DCTRAFO_PROP 1e5
#define USET_OFFSET 94.11//Frequenz=USET[V]*USET_PROP + USET_OFFSET
#define USET_PROP 1.5623027
#define AGILENT_PROP 1e6 //Wert wird in Einheiten von V bermittelt
#define AGILENT_RATIO 24981 //Teilerverhltnis
//#define TREND_SCAL_HIGH 10000
#define COBRA_RAW_TO_NM 1e6
#define ATOS_RAW_TO_NM 1e6
// Labview Data:
#define NR_LV_UNIXTIME 		0
#define NR_LV_TIMERMS 		1
#define NR_LV_VOLTECOOL 	2
#define NR_LV_COBRASETWL 	3
#define NR_LV_ATOSWL 		4
#define NR_LV_ATOSINTENSITY 5
#define NR_LV_ATOSNUMIFM 	6
#define NR_LV_STATFLUKE 	7
#define NR_LV_STATATOS 		8
#define NR_LV_COBMOTHI 		9
#define NR_LV_COBMOTLO 		10
#define NR_LV_COBNUMLOOP 	11
#define NR_LV_COBSTAT 		12
#define NR_LV_ATOSCNTR 		13
#define NR_LV_COBFROMWL		14
#define NR_LV_COBTOWL		15
#define NR_LV_COBINCREM		16
#define NR_LV_COBSTEPNR		17
#define NR_LV_HEINZREGVOLT	18
#define NR_LV_STATAGILENT	19
// total number
#define TOTAL_LV_NUMBERS	20 //totale Anzahl der LV-Daten longs
const std::string lvNames[] = {
		"UTC timecode",
		"ms-Timer",
		"Ecooler Voltage",
		"Cobra Set Wavelength",
		"WS7 measured Wavelength",
		"WS7 Frequency",
		"WS7 Signal Value",
		"Status Keysight",
		"WS7 Temperature",
		"Cobra motor position upper part",
		"Cobra motor position lower part",
		"Cobra number of all loops",
		"Status Cobra",
		"WS7 Pressure",
		"Cobra Start Wavelength Setting",
		"Cobra Stop Wavelength Setting",
		"Cobra Increment Setting",
		"Cobra current Loop Number",
		"Heinzinger regulator voltage",
		"Status Agilent 3458"
};
//TString lvTestString;
const std::string lvUnits[TOTAL_LV_NUMBERS] = {
		"[s]",
		"[ms]",
		"[mu V]",
		"[fm]",
		"[fm]",
		"[percent]",
		"[number]",
		"[state]",
		"[state]",
		"upper part of position",
		"lower part of position",
		"[number]",
		"[state]",
		"[number]",
		"[fm]",
		"[fm]",
		"[fm]",
		"[number]",
		"[V]",
		"[state]"
};
const std::string channelNames[MAX_SCA] = {
		"XUV_Back",			// 0	Analog, TDC
		"XUV_Anode",		// 1 	Analog, TDC
		"XUV_Anode_delay",// 2 	Analog, TDC Time delta between Ch2 and Ch1 represents pulse height
		"PMT_South",		// 3 	Analog, TDC
		"PMT_middle",		// 4 	Analog, TDC
		"PMT_North",		// 5 	Analog, TDC
		"PMT_Cu",			// 6 	Analog, TDC
		"PMT_ParDet_South",	// 7 	Analog, TDC
		"PMT_ParDet_North",	// 8 	Analog, TDC
		"Q-switch_500us",	// 9 	Analog, TDC 
		"Q-switch_1ms",		// 10	Analog, TDC 
        "PMT_north_gated",	// 11	Analog,TDC, PMT North 1 ms gated 
		"PMT_middle_gated",	// 12	Analog, TDC, PMT Middle 1 ms gated
		"PD NO",		    // 13	Analog, TDC PD NO = fast Photodiode Nord Ost
		"PD SO",		    // 14	Analog, TDC PD SO = fast Photodiode Sd Ost
		"Q-switch",			// 15	Analog, TDC
		"DC Trafo",			// 16	Digital, Scaler
		"Injection",		// 17	Digital, Scaler
		"U E-cooler",		// 18	Digital, Scaler
		"I E-cooler",		// 19	Digital, Scaler
		"P gun",			// 20	Digital, Scaler
		"P coll",			// 21	Digital, Scaler
		"Bunching HF South",// 22	Digital, Scaler
		"1 MHz clock",		// 23	Digital, Scaler
		"Empty", 			// 24	Digital, Scaler 
		"Empty",			// 25	Digital, Scaler 1MHz stobed coincidence
		"Empty",			// 26	Digital, Scaler 1MHz stobed coincidence
		"Dye Out",			// 27	Digital, Scaler 1MHz stobed coincidence
		"Empty",			// 28	Digital, Scaler 1MHz stobed coincidence
		"Detector In (1MHz)",		// 29	Digital, Scaler 1MHz stobed coincidence 
		"Laser Scan Compl.",	// 30	Digital, 
		"Start Next Laser Step"	// 31	Digital,

};




#define MAX_STEP  12
#define    CH_N     0        //0
#define    CH_XUV 	1	// 1 	Analog, TDC
#define    CH_SOUTH 2	// 2 	Analog, TDC
#define    CH_MIDDLE 3	// 3 	Analog, TDC
#define    CH_NORTH 4	// 4 	Analog, TDC
#define    CH_CU 5		// 5 	Analog, TDC
#define    CH_1MHZ 6      // 6
#define    CH_ALL 7  
#define    CH_SA_SZ 8  
#define    CH_NA_SZ 9 
#define    CH_NORTH_GATED 10
#define    CH_MIDDLE_GATED 11 

const std::string stepNames[MAX_STEP] = {
    "N",        //0
    "XUV",		// 1 	Analog, TDC
    "SOUTH",	// 2 	Analog, TDC
    "MIDDLE",	// 3 	Analog, TDC
    "NORTH",	// 4 	Analog, TDC
    "CU",		// 5 	Analog, TDC
    "1MHZ",      // 6
    "ALL",     // 7 presently Middle and North    
    "SA_SZ",      // 8    
    "NA_SZ",      // 9    
    "NORTH_GATED",      // 10
    "MIDDLE_GATED",      // 11
};
    
    
/*
 * 	 * UTC,
		 * TIMERMS,
		 * VOLTECOOL,
		 * COBRASETWL,
		 * ATOSWL
		 * ATOSINTENSITY
		 * ATOSNUMIFM
		 * STATAGILENT
		 * STATATOS
		 * COBMOTHI
		 * COBMOTLO
		 * COBNUMLOOP
		 * COBSTAT
		 * ATOSCNTR
 */




const static unsigned int PMTs[]= {0};
#define PMT_NUMBER (UInt_t)(sizeof(PMTs)/sizeof(*PMTs))

#endif /* TLDAINPUTDESCRIPTION_H_ */
