// $Id: TLdaParameter.cxx 506 2009-11-06 08:07:59Z linev $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TLdaParameters.h"
#include "Riostream.h"
#include <math.h>

using std::cout;
using std::endl;

//-----------------------------------------------------------
TLdaParameters::TLdaParameters() :
		TGo4Parameter() {
       pusTimeAxisForLaserPulse = 1;
      // 1 : use Qswtich
      // 2 : use PD_SO
    pfUcool = 215000;
    pfRevolutionFrequency = 2.025e6; // in Hz
    piCSReductionFactor = 100; // Reduction Factor for CS
    piNOfCSForValidTimeRange = 10000;
    pfOhmlabsDividerRtatio = 248517.0;       
    piNOfLaserSteps = 44;           
    pfSetWavelengthLow = 601.7;
    pfSetWavelengthHigh = 601.9;      
    piPhaseOffset = 0;      
    pusFoldStepSpectra = 0;
    piLaserStep = 0;

}
//-----------------------------------------------------------
TLdaParameters::TLdaParameters(const char* name) :
		TGo4Parameter(name) {
   
    pusTimeAxisForLaserPulse = 1;
      // 1 : use Qswtich
      // 2 : use PD_SO
    pfUcool = 215000;
    pfRevolutionFrequency = 2.025e6; // in Hz
    piCSReductionFactor = 100; // Reduction Factor for CS
    piNOfCSForValidTimeRange = 10000;
    pfOhmlabsDividerRtatio = 248517.0;
    piNOfLaserSteps = 44;       
    pfSetWavelengthLow = 601.7;
    pfSetWavelengthHigh = 601.9;  
    piPhaseOffset = 0;
    pusFoldStepSpectra = 0; 
    piLaserStep = 0;

}
//-----------------------------------------------------------
TLdaParameters::~TLdaParameters() {
}
//-----------------------------------------------------------
Int_t TLdaParameters::PrintParameter(Text_t * n, Int_t) {
	cout << "Parameter " << GetName() << ":" << endl;
//	cout << "  offset: \t\t\t\t\t" << offset << endl;
//	cout << "  period: \t\t\t\t\t" << period << endl;    
    cout << "  TimeAxisForLaserPulse: \t\t\t\t " << pusTimeAxisForLaserPulse <<endl;;
      // 1 : use Qswtich
      // 2 : use PD_SO
   cout << "  Set Value Cooler Voltage: \t\t\t" << pfUcool << endl;
   cout << "  Ohmlabs Divider Ratio: \t\t\t\t" <<pfOhmlabsDividerRtatio << endl;
   cout << "  Ion Bunch frequency (RF): \t\t\t" << pfRevolutionFrequency <<endl;
    cout << "  Reduction Factor for Common Stop:\t\t\t" <<  piCSReductionFactor << endl;
    cout << "  Low wavelength value of scan range: \t\t\t" <<  pfSetWavelengthLow << endl;    
    cout << "  High wavelength value of scan range: \t\t" <<  pfSetWavelengthHigh << endl;
    cout << "  Number of LaserSteps per Scan: \t\t\t"  << piNOfLaserSteps << endl;    
    cout << "  Fold LaserStepSpectra (accumulate): \t\t\t" << pusFoldStepSpectra <<endl;

    cout << "  Current Laser Step: \t\t\t\t" <<  piLaserStep  << endl;
    cout << "  Phase Offset (TDC Bunches) in Bins: }\t\t"  << piPhaseOffset << endl;
    cout << "  CS freqeuncy = RF / Reduc Factor:\t\t\t" << (pfRevolutionFrequency / piCSReductionFactor) << endl;    
    cout << "  Time between CS :\t\t\t\t" << ( piCSReductionFactor / pfRevolutionFrequency)<< endl;
    cout << "  Number of CS after Laser Shot to include in Analysis: \t" << piNOfCSForValidTimeRange << endl;
    cout << "  This corresponds to about \t" << piNOfCSForValidTimeRange * ( piCSReductionFactor / pfRevolutionFrequency) << " s \t after the Laser Shot" << endl;
//	cout << "  peakCenter=" << peakCenter << endl;
//	cout << "  peakWidth=" << peakWidth << endl;
//	cout << "Rest of Parameters not showed here" << endl;
	return 0;
}
//-----------------------------------------------------------
Bool_t TLdaParameters::UpdateFrom(TGo4Parameter *pp) {
	TLdaParameters* from = dynamic_cast<TLdaParameters*>(pp);
	if (from == 0) {
		cout << "Wrong parameter object: " << pp->ClassName() << endl;
		return kFALSE;
	}
	cout << endl;
	cout << "Updating parameter " << GetName() << " values!" << endl;
    cout << "Old Paramer Values ..." << endl;
    cout << endl;
    
    PrintParameter(0,0);    
	
//    offset = from->offset;
//	period = from->period;
    pusTimeAxisForLaserPulse = from->pusTimeAxisForLaserPulse;
      // 1 : use Qswtich
      // 2 : use PD_SO
    pfUcool = from->pfUcool;
    pfRevolutionFrequency = from->pfRevolutionFrequency;
    piCSReductionFactor = from->piCSReductionFactor;
    piNOfCSForValidTimeRange = from->piNOfCSForValidTimeRange;
    pfOhmlabsDividerRtatio = from->pfOhmlabsDividerRtatio;
    piNOfLaserSteps = from->piNOfLaserSteps;    
    pfSetWavelengthLow = from->pfSetWavelengthLow;
    pfSetWavelengthHigh = from->pfSetWavelengthHigh; 
    piPhaseOffset = from->piPhaseOffset;    
    pusFoldStepSpectra = from->pusFoldStepSpectra;
    piLaserStep =  from->piLaserStep;

    cout << endl;
    cout<< "=============================================="<< endl;
    cout << "***  Updated Paramer Values ..." << endl <<endl;   
    PrintParameter(0,0);   
    

    
	return kTRUE;
}

