// $Id: TLdaUnpackProc.cxx 520 2009-11-10 16:15:53Z goofy $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TLdaUnpackProc.h"
#include "TGo4UserException.h"

#include "Riostream.h"
#include <time.h>
#include "TROOT.h"
#include "TSystem.h"


#include "TH1.h"
#include "TH2.h"
#include "TCutG.h"

#include "TGo4WinCond.h"
#include "TGo4PolyCond.h"
#include "TGo4CondArray.h"
#include "TGo4Picture.h"
#include "TGo4MbsFile.h"
#include "TGo4Analysis.h"
#include <TGo4Analysis.h>

#include "TLdaParameters.h"
#include "TLdaUnpackEvent.h"
#include "TLdaAnlEvent.h"

// Sergey Linev 28.02.14

#include <vector>
#include <ctime>

//-----------------------------------------------------------
TLdaUnpackProc::TLdaUnpackProc() :
		TGo4EventProcessor() {
}
//-----------------------------------------------------------
TLdaUnpackProc::TLdaUnpackProc(const char* name) :
		TGo4EventProcessor(name) {
	cout << "**** TLdaUnpackProc: Create" << endl;

	//// init user analysis objects:
	fPar = (TLdaParameters *) GetParameter("TimeParameters");

    Init();
    InitSpectra();
    InitConditions();
    InitLvSpectra();
    
	
	firstRun = 0;
	firstRunForTimebase = 0;




	hitCtr = 0;
	tsCS_compare[0] = 0;
	tsCS_compare[1] = 0;
	overflow = 0;
}
//-----------------------------------------------------------
TLdaUnpackProc::~TLdaUnpackProc() {
	cout << "**** TLdaUnpackProc: Delete" << endl;
}
//-----------------------------------------------------------
Bool_t TLdaUnpackProc::BuildEvent(TGo4EventElement* dest) {
	Bool_t isValid = kFALSE; // validity of output event

	//TGo4MbsEvent* inE = (TGo4MbsEvent*) GetInputEvent(); // from this
	//TLdaUnpackEvent* outE = (TLdaUnpackEvent*) dest;
    Zero(); 
	
    inE = (TGo4MbsEvent*) GetInputEvent();
    outE = (TLdaUnpackEvent*) dest;

	if (inE == 0) {
		cout << "LdaUnpackProc: no input event !" << endl;
		outE->SetValid(isValid); // to store or not to store
		// default calling Fill method will set validity of out_evt to return value!
		return isValid;
	}
	if (inE->GetTrigger() > 11) {
		cout << "**** LdaUnpackProc: Skip trigger event" << endl;
		outE->SetValid(isValid);
		return isValid;
	}


  if (inE->GetTrigger() <= 8 && (inE->GetDlen() / 2 + 2) <20000 ) {

	isValid = kTRUE;

	inE->ResetIterator();   
    s_bufhe* bufheader = inE->GetMbsBufferHeader();

	TGo4MbsSubEvent* psubevt(0);
//	psubevt = inE->NextSubEvent();
    
    while ((psubevt = inE->NextSubEvent()) != 0) // subevent loop
    {
    
//    cout << "MBS Time: " << bufheader->l_time[0] << " : " << bufheader->l_time[1] << endl;

	//UInt_t *pdata;
    
	pdata = (UInt_t *) psubevt->GetDataField();
    Int_t lwords = psubevt->GetIntLen();

//////////////////////////////////
	outE->SetValid(isValid); // to store or not to store
	// default calling Fill method will set validity of out_evt to return value!

	//event Nr and time extraction from header
	outE->eventCount = inE->GetCount();
    
    outE->eventTime = (time_t)(bufheader->l_time[0]); // seconds since epoch 1970
	outE->eventTimeMilliSec = (Int_t)(bufheader->l_time[1]); // microseconds since eventTime
	//if (inE)
	outE->eventSizeMbs = inE->GetDlen() / 2 + 2; // total longwords (found in TLdaAnalysis.cxx)

//////////////////////////////////	
	
	ReadLmdFileToArrays();
    
 //   cout << "Common Stop Counter: " << iCSCounter  << "\tCommom Stop Scaler " << iCSScaler << endl;
 
//////////////////////////////////	
    
    FillLVRawDataSpectra();
	
//////////////////////////////////	 
 //cout <<  << uiMyCounter;
  }


} else { cout << " LARGE EVENT - DID NOT PROCESS - SIZE:" << (inE->GetDlen() / 2 + 2) << endl;}    
    
	return isValid; // this will overwrite out_evt->SetValid
// except one would have a Fill method implemented in the output event class!
// In this case the return value can be handled differently there.

}

//1000.0*(fRnd->Rndm()-0.5)


void TLdaUnpackProc::FillLVRawDataSpectra()
{
    if (uiLvUnixTime) lvUnixTime_raw->Fill(uiLvUnixTime); //0
    if (uiLvTimerMS ) lvTimerMS_raw->Fill(uiLvTimerMS);  //1
    if (uiLvVoltEcool) lvVoltEcool_raw->Fill(uiLvVoltEcool*fPar->pfOhmlabsDividerRtatio); //2 
    if (uiLvCobraSetWL) lvCobraSetWL_raw->Fill(uiLvCobraSetWL); //3
    if (uiLvAtosWl) lvAtosWl_raw->Fill(uiLvAtosWl+10000.0*(fRnd->Rndm()-0.5)); //4 
    if (uiLvAtosInt) lvAtosInt_raw->Fill(uiLvAtosInt); //5
    if (uiLvAtosNumInt) lvAtosNumInt_raw->Fill(uiLvAtosNumInt); // 6
    if (uiLvFlukeStatus) lvFlukeStatus_raw->Fill(uiLvFlukeStatus); // 7
    if (uiLvAtosStatus) lvAtosStatus_raw->Fill(uiLvAtosStatus); // 8
    if (uiLvCobraMotorHi) lvCobraMotorHi_raw->Fill(uiLvCobraMotorHi); //9
    if (uiLvCobraMotorLo) lvCobraMotorLo_raw->Fill(uiLvCobraMotorLo); //10
    if (uiLvCobraNumLoop) lvCobraNumLoop_raw->Fill(uiLvCobraNumLoop); //11
    if (uiLvCobraStatus) lvCobraStatus_raw->Fill(uiLvCobraStatus); //12
    if (uiLvAtosCounter) lvAtosCounter_raw->Fill(uiLvAtosCounter); //13
    if (uiLvCobraFromWL) lvCobraFromWL_raw->Fill(uiLvCobraFromWL); // 14
    if (uiLvCobraToWL) lvCobraToWL_raw->Fill(uiLvCobraToWL);  //15
    if (uiLvCobraIncrement) lvCobraIncrement_raw->Fill(uiLvCobraIncrement); //16
    if (uiLvCobraStepNr) lvCobraStepNr_raw->Fill(uiLvCobraStepNr); // 17
    if(uiLvHeinzRegVolt) lvHeinzRegVolt_raw->Fill(uiLvHeinzRegVolt); //18
    if (uiLvAgilentStatus) lvAgilentStatus_raw->Fill(uiLvAgilentStatus); // 19
};

void TLdaUnpackProc::ReadLmdFileToArrays()
{
	// Scaler
	for (int i = 0; i < MAX_SCA; i++) {
		long temp, ch, data;
		temp = *pdata++;
		data = temp & 0xfffff;
		ch = (temp & 0xff000000) >> 24;
		//	    printf("scal :raw x%x :  ch %d : data %d \n", temp, ch,  data);
		//     vpmv5_scaler[ch]->Fill(data);
//ooooo		outE->scalerSubEvent[ch] = data; //here, "i" was used instead of "ch". What is the advantage?
        if(i==ch)
        {
            iaScalSE[i] = data;
            //ooooo		
            outE->scalerSubEvent[ch] = data;
        } else {cout << "Vuprom Scaler Channel Mismatch" <<endl;}

        if (data !=0) rawScaler[ch]->Fill(data);
	}
//	int reserveSize = 26624;  // in hex 0x6800 
    //C Brandau: 2022 - The max size here is aligned to the max buffer size of the Vuprom TDC, cf. Doku of Vuprom TDC.
    
//     // TO_DO: Check if size  of TDC data is 0x6800 (==buffer full) and provide special handling for full buffers...
    
    //Theoretically up to 16*16*0,005*20000=25600 (TDC channels *multihit*event duration*common stop rate) --- C. Brandau 2022: NO ---- WRONG, use max. size of TDC data buffers to avoid constant resizing (crash ?), in particular, after long pauses / full buffers...
	// Alternatively, the requied reserved size can never be larger than 
	// the MBS even size, please see below....
	// The vectors below can grow dynamically for large chunks of data 
	// however, for best performance the vectors should a) reserve memeory for the maximum amount of data of the Vuprom (cf. doku, still to do), and vectors might be created in the constructor of the class, and memory reserved, but size of the vectors resetted with vector.clear() . which removes all elements, vector.size() becomes 0, but the memory still remains allocated (due to v.reserve(full_size_of_VUprom_TDC_Buffer)....
	// Sergey Linev 28.2.14; modified by Matthias
    
	//std::vector<long> vIndex1, vIndex2, vTDCdata, vTimestampCS;
    
    // C Brandau 20222: long on most platforms is 8 Byte (and size of variables in C is platform dependent !!), Scaler data is 32 Bit unsigned - use fixed size variable such as uint32_t or ROOTs UInt_t (32Bit)
//	std::vector<UInt_t> vIndex1, vIndex2, vTDCdata, vTimestampCS;
//   std::vector<Long64_t> vCSCounter;
//  int reserveSize = 26624;  // in hex 0x6800 
//	vIndex1.reserve(reserveSize); // preallocates size.
//	vIndex2.reserve(reserveSize); // preallocates size
//	vTDCdata.reserve(reserveSize); // preallocates size
//	vTimestampCS.reserve(reserveSize); // preallocates size;
//  vCSCounter.reserve(reserveSize);

	//_________________________________________________
	//Attention, what follows is pasted and not entirely adjusted!
	//_________________________________________________
	// TDC
    //C Brandau 2022:  The routines should account for full single and for full double buffers, cf doku of Vuprom TDc firmware. In addition, all "static" arrays must have the max size for the full two bffers.
    // It is noted that full buffers are valid data, well, until the buffer is full.
    // In addition, the long waiting at the beginning of a start acq might lead to full buffers (both buffers.....). It has to be checked that afte a "Trigger == 14! (== start acq) BOTH buffers have been read and cleared - and not just the first or even the the wring buffer....
    
    
	long header, mbsEventSize, hitsPerCh[MAX_TDC] = { };
	long buffer;
	long n_cstop = 0;
	long n_hit = 0;
	long n_hit_in_CS = 0;
    long n_hit_stop = 0;
	header = *pdata++;
	if ((header & 0xc0000000) != 0xc0000000) {
		printf("Header is not existing or broken!!! :  %x :\n", (int) header);
	} else {
		mbsEventSize = 0xfffffff & header;// mbsEventSize is a  misleading variable name cf. doku of Vuprom modle):
		//This is the pure VupROM TDC Data + Timestam, aka without the scaler data.... (and oter modules)
		
		//probably similar to fMbsEvent->GetDlen() / 2 + 2 - NOOOO
		//    modid=(0xff000&header)>>12;

		//    if(size>0 && size < MAX_TDC*TDC_MH ){
	
		if (mbsEventSize > 0 && mbsEventSize < MAX_MEM) {
			/*  for(int n=0;n<MAX_TDC;n++){
			 outEventSize[n]=outE->tdcSubEvent[n].GetSize();//taking care of the size of the TArrayI
			 //   printf("Size measurement worked, here channel and size :%x %x \n", (int)n, (int)outEventSize[n]);//used to debug
			 }*/// I had tried to make this Proc more efficient by putting the GetSize here, but it is easier to put this into the next for loop
			for (int i = 0; i < mbsEventSize; i++) {
				long temp;
				temp = *pdata++;
				if (inE->GetTrigger() == 1) {
					if ((temp & 0xc0000000) == 0x00000000) {//here, we look for TDC data
						long ch, mulhit, cstop_check, data;
						n_hit++;
						n_hit_in_CS++;
                        n_hit_stop++;
						data = temp & 0xffff;
						mulhit = (temp & 0xf0000) >> 16;
						ch = (temp & 0xf00000) >> 20;
						cstop_check = (temp & 0xf000000) >> 24;

//						//here starts Linevs suggestion (from 2010)
//						outEventSize[ch] = outE->tdcSubEvent[ch].GetSize();	//taking care of the size of the TArrayI
//						outE->tdcSubEvent[ch].Set(outEventSize[ch] + 1);//increasing the subarray size
//						outE->arrTimeStampCS[ch].Set(outEventSize[ch] + 1); // Sergey - increase
//						outE->tdcSubEvent[ch][outEventSize[ch]] = data;
//						outE->arrTimeStampCS[ch][outEventSize[ch]] = 0; // Sergey - set 0
//						//here ends Linevs suggestion (from 2010)

						// Sergey Linev - remember both indexes
						vIndex1.push_back(ch);
						vIndex2.push_back(hitsPerCh[ch]++);
						vTDCdata.push_back(data);

						/* if(ch==1){
						 printf("size1 %x mulhit1 %x diff1 %x \n", (int)outEventSize[ch],(int)mulhit, (int)(outEventSize[ch]-mulhit));
						 }*/						//used for debugging
						rawTdc[ch]->Fill(data);
					} else if ((temp & 0xc0000000) == 0xc0000000) {	//here, we look for the first footer{
						long timestampCSlocal = temp & 0x3fffffff;		//30 bit
						for (int i = 0; i < n_hit_in_CS; i++) {
							vTimestampCS.push_back(timestampCSlocal);
						}
						n_hit_in_CS = 0;
						//	  printf("timestamp :raw x%x : time stamp %d  \n", temp, timestamp);
						timeStampCSRaw->Fill(timestampCSlocal);
				// 		outE->timeStampCSEvent = timestampCSlocal;//vermutlich kann hier der Wert mehrfach berschrieben werden.
					} else if ((temp & 0xc0000000) == 0x40000000) {	//here, we look for the second footer
						long refclk;
						refclk = temp & 0xfffffff;
                        iRefClkScaler = refclk-1;
						buffer = (temp & 0x20000000) >> 29;
                        spRefClockScaler->Fill(iRefClkScaler);
						// vpmv5_tdc_rc->Fill( refclk );
					} else if ((temp & 0xc0000000) == 0x80000000) {	//here, we look for the third footer
						long cstop, nhit;
						cstop = temp & 0xfffff;
                        iCSScaler = cstop; //There ist a diffrence of 1 for the two CS counters ???
                        spRefCSCounter->Fill(iCSScaler);
                        if(iCSScaler!=iCSScalerOld) 
                        {
                           iCSCounter++; //counts up for next cs (>) and for next readout trigger (<) after which iCSScler is resetted to 0;
                        }
                        iCSScalerOld = iCSScaler;
                        
                        for (int i = 0; i < n_hit_stop; i++) {
						     vCSCounter.push_back(iCSCounter);
						}
                        n_hit_stop = 0;
                        
						nhit = (temp & 0x1ff00000) >> 20;
						if (cstop != n_cstop)
							printf("TDC scaler borken!!! :  0x%x : %d  %d  \n",
									(int) temp, (int) cstop, (int) n_cstop); //introduced two more typecasts
						if (nhit != n_hit)
							printf("TDC hit broken!!! :  0x%x : %d  %d   \n",
									(int) temp, (int) nhit, (int) n_hit); //introduced two more typecasts
						//   		  vpmv5_tdc_nh[0]->Fill( nhit ); // online
						//   		  vpmv5_tdc_nh[1]->Fill( n_hit ); //offlin
						//   		  //	  vpmv5_tdc_nh_2d->Fill( nhit, n_hit );
						//   		  vpmv5_tdc_cs->Fill( cstop );
						n_cstop++;
						n_hit = 0;
					} else {
						printf("TDC data borken!!! :  %x :\n", (int) temp);
					}
				}
			}
			if (buffer == 0) {
				//   		vpmv5_tdc_bs[0]->Fill( size );
			} else {
				//   		vpmv5_tdc_bs[1]->Fill( size );
			}

			for (int i = 0; i < MAX_TDC; i++) {
                
                aaiTdcSE[i].Set(hitsPerCh[i]);
				outE->tdcSubEvent[i].Set(hitsPerCh[i]);	//increasing the subarray size
                //cout << "Hits " <<  hitsPerCh[i] << endl;
       //         aaiTimeStampCS[i].Set(hitsPerCh[i]);
//				outE->arrTimeStampCS[i].Set(hitsPerCh[i]);//increasing the subarray size 
                outE->arrCSCounter[i].Set(hitsPerCh[i]);
				hitsPerCh[i] = 0;
			}
			// Sergey Linev - set time stamp for all channels
			for (unsigned cnt = 0; cnt < vIndex1.size(); cnt++) {
                 
                aaiTdcSE[vIndex1[cnt]][vIndex2[cnt]] = vTDCdata[cnt];
				
                outE->tdcSubEvent[vIndex1[cnt]][vIndex2[cnt]] = vTDCdata[cnt];
                
                outE->arrCSCounter[vIndex1[cnt]][vIndex2[cnt]]= vCSCounter[cnt];
                
				tsCS_compare[hitCtr % 2] = vTimestampCS[cnt];
				if (tsCS_compare[(hitCtr + 0) % 2]
						< tsCS_compare[(hitCtr + 1) % 2]) {
					overflow++;
					printf("CStimestamp overflow:%u hit ctr: %u \n",overflow,hitCtr);
					fflush(stdout);
				}
				
//				aaiTimeStampCS[vIndex1[cnt]][vIndex2[cnt]] =
//						tsCS_compare[hitCtr % 2] + overflow * THIRTY_BITS;
			
//				outE->arrTimeStampCS[vIndex1[cnt]][vIndex2[cnt]] =						tsCS_compare[hitCtr % 2] + overflow * THIRTY_BITS;
			}
			vIndex1.clear();
			vIndex2.clear(); // empty channels
			vTDCdata.clear();
			vTimestampCS.clear();

		} else if (mbsEventSize == 0x1000) {
			printf("No common stop to TDC!! :  %x :\n", (int) header);
		} else if (mbsEventSize != 0) {
			printf("TDC data size error !! :  %x :\n", (int) header);
		}
	}

    
// pass LabView Data to event
	for (Int_t i = 0; i < TOTAL_LV_NUMBERS; i++) {
       // uiaEventLv[i] = *(pdata + i);     
        uiaEventLv[i] = *pdata++;
      //  uiLvVoltEcool = uiLvVoltEcool*fPar->pfOhmlabsDividerRtatio;// correction of uiaEventLv[2] via anonymous union, see header file
		outE->eventLv[i] = uiaEventLv[i];
   
	}

//set starting time; later this shouldnt be here in Unpack!!
//	if (lvUnixTime != 0) {printf(
//					"UnixTime : %u, ms Timer: %u, voltEcool : %u, CobraSetWL : %u, AtosWL : %u, \n",
//					(unsigned int) lvUnixTime, (unsigned int) lvTimerMS,
//					(unsigned int) lvVoltEcool, (unsigned int) lvCobraSetWL,
//					(unsigned int) lvAtosWl);
//	fflush (stdout);
//	}
	if (lvUnixTime != 0) {
		if (((firstRunForTimebase++) % 1000) == 0) {
			if (firstRunForTimebase == 0) {
				lvUnixTime_base = lvUnixTime;
//				printf("First socket numbers: \n");
			}

		}
	}
// fill histograms with data from LabView
// lvUnixTime_raw	->Fill(lvUnixTime-lvUnixTime_base);
    
    
};

void TLdaUnpackProc::InitSpectra()
{
    // create histograms
//	Double_t scal_low = -0.5;
//	Double_t scal_high = 20000.5;
//	Int_t scal_div = 20001;
//
//	Double_t tdc_low = -0.5;
//	Double_t tdc_high = 65536.5;
//	Int_t tdc_div = 65537;

	char tmpPath[200];
	char tmpTitle[200];

	for (Int_t i = 0; i < MAX_SCA; i++) {
		sprintf(tmpPath, "0_Raw/Scaler/S%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "Scal_%i_%s", i, channelNames[i].c_str());
		rawScaler[i] = MakeTH1('I', tmpPath, tmpTitle, scal_div, scal_low,
				scal_high, "counts per readout", "occurences");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "0_Raw/TDC/T%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "TDC_%i_%s", i, channelNames[i].c_str());
		rawTdc[i] = MakeTH1('I', tmpPath, tmpTitle, tdc_div, tdc_low, tdc_high,
				"time till common stop [bin]", "hits");
	}


    spRefClockScaler = MakeTH1('I', "0_Raw/RefClockScaler",
			"RefClockScaler", 65537, -0.5, 65536.5,
			"RefClockScaler", "hits");
    spRefCSCounter = MakeTH1('I', "0_Raw/CSCounter",
			"CSCounter", 65537, -0.5, 65536.5,
			"CSCounter", "hits");

    timeStampCSRaw = MakeTH1('I', "0_Raw/tdc_timestamp",
			"vpmv5_tdc_timestamp", 20000, 0, 2000000000,
			"value of common stop timestamp [mu s]", "occurences in 100 ms");
};


void TLdaUnpackProc::InitLvSpectra()
{
    
	//rawScaler     = MakeTH1('I', "Scaler", "LMD Signal", 5000, 0, 5000);
    //_____________________________________________________
// 	histograms for LabView Server Data
//_____________________________________________________
	
    time_t utc_currenttime;
    time_t utc_low;
    time_t utc_high;    
    time_t utc_div;
    time_t utc_diff;
    time_t t1min = 60;
    time_t t1hour = 60*60;  
    time_t t1day = 24*60*60;
    time_t t1week = 7*24*60*60;
    
    Bool_t b1OnlineUseUTCCurrentInSpectra;
    
    b1OnlineUseUTCCurrentInSpectra = kTRUE;
    
    time(&utc_currenttime);
    cout << "UTC current time:" << utc_currenttime << endl;
    
    utc_low = 1649500000;
    if (b1OnlineUseUTCCurrentInSpectra) {utc_low = utc_currenttime;
        utc_low = utc_low - t1week;
    }
    
    utc_diff    =   4*t1week;
    utc_div     =   utc_diff/t1min;
    utc_high    =   utc_low + utc_diff;
    

//    //used for all  Wavelength RawSpectra
//    
//    Int_t iWavelengthSteps = 50001; 
//    Int_t iWavelengthMin = 249995000;
//    Int_t iWavelengthMax = 750005000;
//    
//    //used for all Cooler Voltage RawSpectra
//    
//    Int_t iCoolerHVSteps = 24001;
//    Int_t iCoolerHVMin = -5;
//    Int_t iCoolerHVMax = 240005;
//    
//    ULong_t ui32BitMax = 4294967295;
//    ULong_t ui32BitMin = 0;
//    ULong_t ui32BitSteps = 4096;
    
  
    //4096 (ui12Bit>>1) 
    
	lvUnixTime_raw = MakeTH1('I', "0_Raw/LV/lvUnixTime",
			"Labview_UTC_timestamp", utc_div, utc_low, utc_high,
			"value of LV UTC timestamp [s]", "occurences");

	lvTimerMS_raw = MakeTH1('I', "0_Raw/LV/lvTimerMS",
			"Labview_TimerMs_timestamp", ui32BitSteps, ui32BitMin, ui32BitMax,
			"value of LV TimerMs timestamp [ms]", "occurences"); // Labview counts from 0 to (2^32)-1 continuously,

	lvVoltEcool_raw = MakeTH1('I', "0_Raw/LV/lvVoltEcool",
			"Labview_VoltEcool", iCoolerHVSteps, iCoolerHVMin, iCoolerHVMax,
			"value of LV VoltEcool [7.5 sign.dig]", "occurences");

	lvCobraSetWL_raw = MakeTH1('I', "0_Raw/LV/lvCobraSetWl",
			"Labview_CobraSetWL", iWavelengthSteps, iWavelengthMin, iWavelengthMax,
			"value of LV CobraSetWL [fm]", "occurences");



    
	lvAtosWl_raw = MakeTH1('I', "0_Raw/LV/lvWS7Wl",
			"Labview_WS7WL", iWavelengthSteps, iWavelengthMin, iWavelengthMax,
			"value of LV WS7Wavelength [fm]", "occurences");

	lvAtosInt_raw = MakeTH1('I', "0_Raw/LV/lvWS7Freq",
			"Labview_WS7Freq", iWavelengthSteps, iWavelengthMin, iWavelengthMax,
			"value of LV WS7Frequency", "occurences");

	lvAtosNumInt_raw = MakeTH1('I', "0_Raw/LV/lvAtosNumInt",
			"Labview_AtosInterferometers", 6, -0.5, 5.5,
			"number of used interferometers", "occurences");

	lvAgilentStatus_raw = MakeTH1('I',
			"0_Raw/LV/lvAgilentStatus_raw", "Labview_AgilentStatus",
			3, -0.5, 2.5, "value AgilentStatus", "occurences");

	lvAtosStatus_raw = MakeTH1('I', "0_Raw/LV/lvAtosStatus",
			"Labview_AtosStatus", 3, -0.5, 2.5, "value AtosStatus", "occurences");

	lvCobraMotorHi_raw = MakeTH1('I', "0_Raw/LV/lvCobraMotorHi",
			"Labview_CobraMotorHi", ui32BitSteps, ui32BitMin, ui32BitMax ,
			"value CobraMotorHi", "occurences");

	lvCobraMotorLo_raw = MakeTH1('I', "0_Raw/LV/lvCobraMotorLo",
			"Labview_CobraMotorLo", ui32BitSteps, ui32BitMin, ui32BitMax,
			"value CobraMotorLo", "occurences");

	lvCobraNumLoop_raw = MakeTH1('I', "0_Raw/LV/lvCobraNumLoop",
			"Labview_CobraNumLoop", 51, -0.5, 50.5, "value CobraNumLoop",
			"occurences");

	lvCobraStatus_raw = MakeTH1('I', "0_Raw/LV/lvCobraStatus",
			"Labview_CobraStatus", 65, -0.5, 64.5, "value CobraStatus",
			"occurences");

	lvAtosCounter_raw = MakeTH1('I', "0_Raw/LV/lvAtosCounter",
			"Labview_AtosCounter", ((30 * 3600 * 10) / 300), 0,
			(30 * 3600 * 10), "value AtosCounter", "occurences");
	lvCobraFromWL_raw = MakeTH1('I', "0_Raw/LV/lvCobraFromWL",
			"Labview_CobraFromWavelength", iWavelengthSteps, iWavelengthMin, iWavelengthMax,
			"value CobraFromWavelength", "occurences");
	lvCobraToWL_raw = MakeTH1('I', "0_Raw/LV/lvCobraToWL",
			"Labview_CobraToWavelength", iWavelengthSteps, iWavelengthMin, iWavelengthMax,
			"value CobraToWavelength", "occurences");
	lvCobraIncrement_raw = MakeTH1('I',
			"0_Raw/LV/lvCobraIncrement",
			"Labview_CobraIncrement_raw", 1000, -50, 100050,
			"value lvCobraIncrement", "occurences");

	lvCobraStepNr_raw = MakeTH1('I', "0_Raw/LV/lvCobraStepNr",
			"Labview_CobraStepNr_raw", 401, -0.5, 400.5, "value CobraStepNr",
			"occurences");
	lvHeinzRegVolt_raw = MakeTH1('I', "0_Raw/LV/lvHeinzRegVolt",
			"Labview_HeinzRegVolt", iCoolerHVSteps, iCoolerHVMin, iCoolerHVMax,
			"value of LV HeinzRegVolt [7.5 sign.dig]", "occurences");
	lvFlukeStatus_raw = MakeTH1('I',
			"0_Raw/LV/lvFlukeStatus", "Labview_FlukeStatus",
			3, -0.5, 2.5, "value FlukeStatus", "occurences");
    
};

void TLdaUnpackProc::InitConditions()
{
    
};

void TLdaUnpackProc::Init()
{
    fRnd = new TRandom();
    UInt_t reserveSize = 26624;  // in hex 0x6800 
    vIndex1.reserve(reserveSize); // preallocates size.
	vIndex2.reserve(reserveSize); // preallocates size
	vTDCdata.reserve(reserveSize); // preallocates size
	vTimestampCS.reserve(reserveSize); // preallocates size;
    vCSCounter.reserve(reserveSize);

};
void TLdaUnpackProc::Zero()
{
//    pdata = 0; // Main Data Pointer for LMD Input File
    inE = 0; 
    outE = 0;     
    uiMyCounter = 0; 
    iRefClkScaler = 0;
    
    for(int i=0; i<MAX_SCA; i++){
        iaScalSE[i] = 0;
    }
    
    for (UInt_t i = 0; i < TOTAL_LV_NUMBERS; i++){
        uiaEventLv[i] = 0;
    }; 
    
   //  aaiTdcSE[MAX_TDC]; 
    
    vIndex1.clear(); 
	vIndex2.clear(); 
	vTDCdata.clear();
	vTimestampCS.clear(); 
    vCSCounter.clear();

//	vIndex1.reserve(reserveSize); // preallocates size.
//	vIndex2.reserve(reserveSize); // preallocates size
//	vTDCdata.reserve(reserveSize); // preallocates size
//	vTimestampCS.reserve(reserveSize); // preallocates size;
//  vCSCounter.reserve(reserveSize);

    
 //   iCSScaler = 0;
};
