// $Id: TLdaUnpackProc.h 506 2009-11-06 08:07:59Z linev $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#ifndef TLdaUNPACKPROCESSOR_H
#define TLdaUNPACKPROCESSOR_H

#include "TGo4EventProcessor.h"
#include "TLdaUnpackEvent.h"
#include "TLdaInputDescription.h"
#include "s_filhe_swap.h"
#include "s_bufhe_swap.h"

#include "TGo4MbsEvent.h"
#include <TRandom.h>
#include <TArrayL64.h>

class TLdaParameters;
class TLdaUnpackEvent;

class TLdaUnpackProc: public TGo4EventProcessor {
public:
	TLdaUnpackProc();
	TLdaUnpackProc(const char* name);
	virtual ~TLdaUnpackProc();
	virtual Bool_t BuildEvent(TGo4EventElement* dest);
	TLdaParameters *fPar;
//	Parameter2ndList *fPar2;
private:
    void ReadLmdFileToArrays();
    void FillLVRawDataSpectra();
    void Init();
    void InitSpectra();    
    void InitConditions();
    void InitLvSpectra();
    void Zero();
    
    UInt_t *pdata; // Main Data Pointer for LMD Input File
    
    TGo4MbsEvent* inE; 
    TLdaUnpackEvent* outE; 
    TRandom *fRnd;
    
    
const Double_t scal_low = -0.5;
const Double_t scal_high = 20000.5;
const Int_t scal_div = 20001;
//
const Double_t tdc_low = -0.5;
const Double_t tdc_high = 32768.5;
const Int_t tdc_div = 32769;

//    //used for all  Wavelength RawSpectra
//    
const    Int_t iWavelengthSteps = 50001; 
const    Double_t iWavelengthMin = 249995000;
const    Double_t iWavelengthMax = 750005000;
//    
//    //used for all Cooler Voltage RawSpectra
//    
const    Int_t iCoolerHVSteps = 240001;
const    Double_t iCoolerHVMin = -0.5;
const    Double_t iCoolerHVMax = 240000.5;
//    
const    ULong_t ui32BitMax = 4294967295;
const    ULong_t ui32BitMin = 0;
const    ULong_t ui32BitSteps = 4096; 
     
	TH1 *rawScaler[MAX_SCA];
	TH1 *rawTdc[MAX_TDC];
 //   TH1 *rawLV[TOTAL_LV_NUMBERS];
	
    TH1 *timeStampCSRaw;
	UInt_t timeStampCS;
    
    Int_t iRefClkScaler=0;
    Int_t iCSScaler=0;
    Int_t iCSScalerOld=0;
    
    
   
    Long64_t lAbsoluteTimeStamp=0; // in VupromTDC Units 10/3ns
    UInt_t iCSCounter=0;    
   // Long64_t iCSCounterOld=0;
     
   //    f_ecm = f_ecm+fParam1->pf_random*(fRnd->Rndm()-0.5*fParam1->pf_random);  
   //value = (fRnd->Rndm()-0.5); verschmieren ber einen Kanal


    
    TH1 *spRefClockScaler;
    TH1 *spRefCSCounter;
    
    std::vector<UInt_t> vIndex1, vIndex2, vTDCdata, vTimestampCS, vCSCounter;
   // std::vector<Long64_t> vCSCounter;
    
    //Alias for Labview Data, can use Loop over Indices AND real name for same data
    union
    {
	    UInt_t uiaEventLv[TOTAL_LV_NUMBERS]; // 20?
	    struct 
	    {
            UInt_t uiLvUnixTime; //0
            UInt_t uiLvTimerMS;  //1
            UInt_t uiLvVoltEcool; //2
            UInt_t uiLvCobraSetWL; //3
            UInt_t uiLvAtosWl; //4
            UInt_t uiLvAtosInt; //5
            UInt_t uiLvAtosNumInt; //6 numifm
            UInt_t uiLvFlukeStatus;  //7
            UInt_t uiLvAtosStatus; //8
            UInt_t uiLvCobraMotorHi; //9
            UInt_t uiLvCobraMotorLo; //10
            UInt_t uiLvCobraNumLoop; //11
            UInt_t uiLvCobraStatus; //12
            UInt_t uiLvAtosCounter; //13
            UInt_t uiLvCobraFromWL;  //14
            UInt_t uiLvCobraToWL;    //15
            UInt_t uiLvCobraIncrement; //16
            UInt_t uiLvCobraStepNr; //17
            UInt_t uiLvHeinzRegVolt; //18
            UInt_t uiLvAgilentStatus; //19
	    };
	};
    
    union
    {
	    UInt_t iaScalSE[MAX_SCA]; //32 
	    struct 
	    {
            Int_t iScXUVBack; //0
            Int_t iScXUVAnode;  //1
            Int_t iScXUVAnodeDelayed;//2
            Int_t iScPMTSouth; //3
            Int_t iScPMTMiddle; //4
            Int_t iScPMTNorth; //5
            Int_t iScCu; //6 numifm
            Int_t iScPartDetSouth;  //7
            Int_t iScPartDetNorth; //8
            Int_t iScDummy1; //9
            Int_t iScDummy2; //10
            Int_t iScDummy3; //11           
            Int_t iScDummy4; //12
            Int_t iScPDNO; //13
            Int_t iScPDSO; //14
            Int_t iScQSwitch;  //15
            Int_t iScDCTrafo;    //16
            Int_t iScInjection; //17
            Int_t iScUCool; //18
            Int_t iScICool; //19
            Int_t iScPGun; //20          
            Int_t iScPCol; //21      
            Int_t iScBunchHFSouth; //22            
            Int_t iSc1MHz; //23
            Int_t iScDummy5; //24
            Int_t iScDummy6; //25           
            Int_t iScDummy7; //26
            Int_t iScLaserBusyStrobe; //27
            Int_t iScDummy8; //28            
            Int_t iScDetInStrobe; //29            
            Int_t iScNewLaserStep; //30            
            Int_t iScEndOfScan; //31
                     
        };
	};
    

 //   union
 //   { 
        TArrayI aaiTdcSE[MAX_TDC]; //TDC subevents	    tdcSE[MAX_TDC]
 //       struct 
//	    {
//            TArrayI aiTdcXUVBack; //0
//            TArrayI aiTdcXUVAnode;  //1
//            TArrayI aiTdcXUVAnodeDelayed;//2
//            TArrayI aiTdcPMTSouth; //3
//            TArrayI aiTdcPMTMiddle; //4
//            TArrayI aiTdcPMTNorth; //5
//            TArrayI aiTdcCu; //6 numifm
//            TArrayI aiTdcPartDetSouth;  //7
//            TArrayI aiTdcPartDetNorth; //8
//            TArrayI aiTdcDummy1; //9
//            TArrayI aiTdcDummy2; //10
//            TArrayI aiTdcDummy3; //11           
//            TArrayI aiTdcDummy4; //12
//            TArrayI aiTdcPDNO; //13
//            TArrayI aiTdcPDSO; //14
//            TArrayI aiTdcQSwitch;  //15        
//        };        
//    };
	
	
    TArrayI aaiTimeStampCS[MAX_TDC]; //timestamp common stop, same structure  
    TArrayL64 alTimeStampCS[MAX_TDC];
    
        //----> kein Double !!!!!!! 
    Double_t tdcHitCount[MAX_TDC];
    
	//Values from Labview Server
	TH1 *lvUnixTime_raw;
	UInt_t lvUnixTime;
	UInt_t lvUnixTime_base;
	TH1 *lvTimerMS_raw;
	UInt_t lvTimerMS;
	TH1 *lvVoltEcool_raw;
	UInt_t lvVoltEcool;
	TH1 *lvCobraSetWL_raw;
	UInt_t lvCobraSetWL;
	TH1 *lvAtosWl_raw;
	UInt_t lvAtosWl;
	TH1 *lvAtosInt_raw;
	UInt_t lvAtosInt;
	TH1 *lvAtosNumInt_raw;
	UInt_t lvAtosNumInt;
	TH1 *lvAgilentStatus_raw;
	UInt_t lvAgilentStatus;
	TH1 *lvAtosStatus_raw;
	UInt_t lvAtosStatus;
	TH1 *lvCobraMotorHi_raw;
	UInt_t lvCobraMotorHi;
	TH1 *lvCobraMotorLo_raw;
	UInt_t lvCobraMotorLo;
	TH1 *lvCobraNumLoop_raw;
	UInt_t lvCobraNumLoop;
	TH1 *lvCobraStatus_raw;
	UInt_t lvCobraStatus;
	TH1 *lvAtosCounter_raw;
	UInt_t lvAtosCounter;
	TH1 *lvCobraFromWL_raw;
	UInt_t lvCobraFromWL;
	TH1 *lvCobraToWL_raw;
	UInt_t lvCobraToWL;
	TH1 *lvCobraIncrement_raw;
	UInt_t lvCobraIncrement;
	TH1 *lvCobraStepNr_raw;
	UInt_t lvCobraStepNr;
	TH1 *lvHeinzRegVolt_raw;
	UInt_t lvHeinzRegVolt;
	TH1 *lvFlukeStatus_raw;
	UInt_t lvFlukeStatus;


	//char	lmdName[100];
	Int_t firstRun;
	Int_t firstRunForTimebase;
	s_filhe* currFileHeader;

	UInt_t hitCtr;//index used to check for overflow
	UInt_t tsCS_compare[2];// compares CS values in order to check for overflow
	UInt_t overflow;

    UInt_t uiMyCounter;

	ClassDef(TLdaUnpackProc,1)
};

#endif //TLdaUNPACKPROCESSOR_H
