// $Id: TModelTemplate.cxx 554 2010-01-22 15:53:21Z linev $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

/* template for user model class
   The only function, which should be redefined by used - UserFunction()
   In given example constructor provides a general interface to create
   several parameters. Amplitude parameters can be add, if necessary.
   Any kind of other constructors with predefine parameters set and names can be invented
   Here user function is Gaussian
*/

#include <math.h>
#include "TString.h"
#include "TMath.h"
#include "TFoldGauss2.h"

TFoldGauss2::TFoldGauss2(const char* iName, Int_t iNPars, Bool_t AddAmplitude) :
   TGo4FitModel(iName,"Gauss folded with two exponential decays",AddAmplitude)
{
   for (Int_t n=0;n<iNPars;n++) {
       TString s("Par"); s+=n;
       NewParameter(s.Data(),"model template parameter",0.);
   }
}

Double_t TFoldGauss2::myErfApprox(Double_t x){//as described in wikipedia
	Double_t a=0.140012;// best approx near zero
	Double_t mySignum=1;
	if (x<0){
		mySignum=-1;
	}
	return mySignum*TMath::Sqrt(1-TMath::Exp(-x*x*((4/TMath::Pi())+a*x*x)/(1+a*x*x)));
}

Double_t TFoldGauss2::SingleFoldGauss(Double_t pos, Double_t lambda, Double_t sigma, Double_t x){
	   x = pos - x;
	   return 0.5*TMath::Exp((0.5 *lambda)*(-2*x + lambda*pow(sigma,2)))*lambda*(1 + TMath::Erf((x - lambda * pow(sigma,2))/(TMath::Sqrt(2)*sigma)));
}

Double_t TFoldGauss2::UserFunction(Double_t* Coordinates, Double_t* Parameters)
{
	Double_t amp1 = Parameters[0];
	Double_t amp2 = Parameters[1];
	Double_t pos = Parameters[2];
	Double_t sigma = Parameters[3];
	Double_t lambda1 = Parameters[4];
	Double_t lambda2 = Parameters[5];

   Double_t x = Coordinates[0];

   return amp1 * SingleFoldGauss(pos,lambda1,sigma,x) + amp2 * SingleFoldGauss(pos,lambda2,sigma,x);
}
