// $Id: TLda3aIntDiffProc.cxx 520 2009-11-10 16:15:53Z goofy $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TLda3aIntDiffProc.h"

#include <math.h>

#include "TH1.h"
#include "TH2.h"
#include "Riostream.h"
#include "TROOT.h"

#include "TGo4WinCond.h"
#include "TGo4Analysis.h"

#include "TGo4Fitter.h"

#include "TLdaAnlEvent.h"
#include "TLda3aIntDiffEvent.h"
#include "TLdaParameters.h"

//-----------------------------------------------------------
TLda3aIntDiffProc::TLda3aIntDiffProc() :
		TGo4EventProcessor() {
}
//-----------------------------------------------------------
TLda3aIntDiffProc::TLda3aIntDiffProc(const char* name) :
		TGo4EventProcessor(name) {
	cout << "**** TLda3aIntDiffProc: Create" << endl;
	//// init user analysis objects:
	// if unpack was enabled, parameters have been printed already
	//   if(TGo4Analysis::Instance()->GetAnalysisStep("Unpack")->IsProcessEnabled())
	//	    gROOT->ProcessLine(".x setparam.C(0)");
	//   else gROOT->ProcessLine(".x setparam.C(1)");

	//This is our parameter bitch! Loaded here for scope reasons

	fPar = (TLdaParameters *) GetParameter("TimeParameters");
	fPar2 = (Parameter2ndList *) GetParameter("ControlParameters");

	scanningCtr = 0;

	char tmpPath[100];
	char tmpTitle[100];

// original code before 5.4.2022
//	int scan_div = 151;
//	int scan_low = -0.5;
//	int scan_high = 150.5;
	
	Int_t scan_div = 151;
	Double_t scan_low = -0.5;
	Double_t scan_high = 150.5;
	
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "step3a_IntDiff/Scan_Int_Diff_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "Scan_Int_Diff_%i_%s", i, channelNames[i].c_str());
		scanIntDiff[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "Laser Step Number",
				"Difference counts in Meas. bunch - Ref. bunch");
		sprintf(tmpPath, "step3a_IntTotal/Scan_Int_Total_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "Scan_Int_Total_%i_%s", i, channelNames[i].c_str());
		scanIntTotal[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "Laser Step Number",
				"Total counts in Meas. bunch + Ref. bunch");
	}

	//	for (int i = 0; i < MAX_TDC; i++) {
	//		tdcConvol[i].Set((Int_t) ceil(fPar->period));
	//	}
	//
	//	char tmpstr[20];
	//
	//	for (Int_t i = 0; i < MAX_TDC; i++) {
	//		sprintf(tmpstr, "TDC_convoluted_%i", i);
	//		convolutedTdcHist[i] = MakeTH1('I', tmpstr, tmpstr, tdc_div, tdc_low,
	//				tdc_high, "convoluted time till common stop [bin]", "hits");
	//	}
}

//-----------------------------------------------------------
TLda3aIntDiffProc::~TLda3aIntDiffProc() {
	cout << "**** TLda3aIntDiffProc: Delete" << endl;

}
//-----------------------------------------------------------
Bool_t TLda3aIntDiffProc::BuildEvent(TGo4EventElement* dest) {
	Bool_t isValid = kFALSE; // validity of output event
	TLdaAnlEvent* in_evt = (TLdaAnlEvent*) GetInputEvent();
	TLda3aIntDiffEvent* out_evt = (TLda3aIntDiffEvent*) dest;

	// laser not switched, event empty
	if ((in_evt == 0) || !in_evt->IsValid()) { // input invalid
		out_evt->SetValid(isValid); // invalid
		return isValid; // must be same is for SetValid
	}
	scanningCtr++;
	cout << "Int/Diff: the " << scanningCtr
			<< ". time, stepNr in input event = " << in_evt->stepNr << endl;
	//Buffer for determining signal strength
	Int_t tempSignalIntDiff[MAX_TDC];
	Int_t tempSignalTotal[MAX_TDC];

	Int_t loBoundMeas = (fPar->peakCenter - fPar->peakWidth);//if modifying these four variables, take care.
	Int_t upBoundMeas = (fPar->peakCenter + fPar->peakWidth);// They have also been copied to AnlProc.cxx
	Int_t loBoundRef = (fPar->peakCenter + (Int_t) floor((fPar->period / 2))
			- fPar->peakWidth);
	Int_t upBoundRef = (fPar->peakCenter + (Int_t) floor((fPar->period / 2))
			+ fPar->peakWidth);
	//do it for all tdc channels
	for (int j = 0; j < MAX_TDC; j++) {
		tempSignalIntDiff[j] = 0;
		tempSignalTotal[j] = 0;

		//sum over all counts in left (=measurement) peak (width is half-width)
		for (int i = loBoundMeas; i <= upBoundMeas; i++) {
			tempSignalIntDiff[j] += in_evt->phaseCounts[j][i];
			tempSignalTotal[j] += in_evt->phaseCounts[j][i];
		}
		//subtract left (=reference) peak
		for (int i = loBoundRef; i <= upBoundRef; i++) {
			tempSignalIntDiff[j] -= in_evt->phaseCounts[j][i];
			tempSignalTotal[j] += in_evt->phaseCounts[j][i];
		}
		if ((fPar2->flagHistStepIntDiff) != 0) {
			scanIntDiff[j]->SetBinContent(in_evt->stepNr+1, tempSignalIntDiff[j]); //take care: Bin=0 is underflow bin, Bin=1 is first data bin. Now, stepNr can be 0, so I add +1
			scanIntTotal[j]->SetBinContent(in_evt->stepNr+1, tempSignalTotal[j]); //take care: Bin=0 is underflow bin, Bin=1 is first data bin. Now, stepNr can be 0, so I add +1
		}
		out_evt->signal[j] = (Int_t) tempSignalIntDiff[j];
		out_evt->signalAndBackground[j] = (Int_t) tempSignalTotal[j];
	}
	out_evt->eventMbsTime = in_evt->eventMbsTime;
	out_evt->eventUsefulLifeTime = in_evt->eventUsefulLifeTime;
	out_evt->stepNr = in_evt->stepNr;
	out_evt->atosAvgWL = in_evt->atosAvgWL;
	out_evt->lastAtosWL = in_evt->lastAtosWL;
	out_evt->cobraSetWL = in_evt->cobraSetWL;
	sprintf(out_evt->lmdName, in_evt->lmdName);
	for (int i = 0; i < MAX_SCA; i++) {
		out_evt->lasingAcc[i] = in_evt->lasingAcc[i];
	}
	for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
		out_evt->lvAvgVal[i]=in_evt->lvAvgVal[i];
	}

	//cout<<"stepNr in IntDiff aus out_evt="<<out_evt->stepNr;
	isValid = kTRUE;

	//continue evil kanevil hack
//	out_evt->signal[12] = in_evt->raw12;
//	out_evt->signal[13] = in_evt->raw13;

	// see comments in UnpackProc
	out_evt->SetValid(isValid);
	return isValid;
}

