// $Id: TLda3aIntDiffProc.cxx 520 2009-11-10 16:15:53Z goofy $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------


#include <math.h>

#include "TH1.h"
#include "TH2.h"
#include "Riostream.h"
#include "TROOT.h"
#include "TMath.h"

#include "TGo4WinCond.h"
#include "TGo4Analysis.h"

#include "TLda3bBunchFitProc.h"
#include "TGo4Fitter.h"

#include "TLdaAnlEvent.h"
#include "TLda3bBunchFitEvent.h"
#include "TLdaParameters.h"
#include "TFoldGauss2.h"


//-----------------------------------------------------------
TLda3bBunchFitProc::TLda3bBunchFitProc() :
	TGo4EventProcessor() {
}
//-----------------------------------------------------------
TLda3bBunchFitProc::TLda3bBunchFitProc(const char* name) :
	TGo4EventProcessor(name) {
	cout << "**** TLda3bBunchFitProc: Create" << endl;
	//// init user analysis objects:
	// if unpack was enabled, parameters have been printed already
	//   if(TGo4Analysis::Instance()->GetAnalysisStep("Unpack")->IsProcessEnabled())
	//	    gROOT->ProcessLine(".x setparam.C(0)");
	//   else gROOT->ProcessLine(".x setparam.C(1)");

	//This is our parameter bitch! Loaded here for scope reasons


	fPar = (TLdaParameters *) GetParameter("TimeParameters");
	fPar2 = (Parameter2ndList *) GetParameter("ControlParameters");
	fittingCtr = 0;

	char tmpPath[100];
	char tmpTitle[100];

	Int_t scan_div = 1000;
	Double_t scan_low = 0.;
	Double_t scan_high = 1000.;

	Int_t tofit_div = 1000;
	Double_t tofit_low = 0.;
	Double_t tofit_high = 1000.;

	fitTimeHist = MakeTH1('D', "Step3b_Fit/TotalTime", "TotalTimePerFit", scan_div,
			scan_low, scan_high, "laser scan step or gui-set time bin",
			"accumulated MBS time elapsed during this step [s]");

	fitLifeTimeHist = MakeTH1('D', "Step3b_Fit/LifeTime", "LifeTimePerFit", scan_div,
			scan_low, scan_high, "laser scan step or gui-set time bin",
			"accumulated useful life time in this bin [s]");

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step3b_Fit/LongContribution/Fit_Amp_Long_%i_%s", i,channelNames[i].c_str());
		sprintf(tmpTitle, "Fit_Amp_Long_%i_%s", i,channelNames[i].c_str());
		fitAmpLong[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "laser scan step or gui-set time bin",
				"Counts in long-lived contribution");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step3b_Fit/ShortContribution/Fit_Amp_Short_%i_%s", i,channelNames[i].c_str());
		sprintf(tmpTitle, "Fit_Amp_Short_%i_%s", i,channelNames[i].c_str());
		fitAmpShort[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "laser scan step or gui-set time bin",
				"Counts in short-lived contribution");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step3b_Fit/Signal/Fit_Amp_Signal_%i_%s", i,channelNames[i].c_str());
		sprintf(tmpTitle, "Fit_Amp_Signal_%i_%s", i,channelNames[i].c_str());
		fitAmpSignal[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "laser scan step or gui-set time bin",
				"Counts in Gaussian signal above reference structure");
	}

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step3b_Fit/Background/Uniform_Background_%i_%s", i, channelNames[i].c_str());
		sprintf(tmpTitle, "Uniform_Background_%i_%s", i,channelNames[i].c_str());
		uniformBackground[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div,
				scan_low, scan_high, "laser scan step or gui-set time bin",
				"Counts in uniform background");
	}

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step3b_Fit/SeeTheFit/to_be_fitted_%i_%s", i, channelNames[i].c_str());
		sprintf(tmpTitle, "to_be_fitted_%i_%s", i,channelNames[i].c_str());
		toBeFitted[i] = MakeTH1('I', tmpPath, tmpTitle, tofit_div, tofit_low,
				tofit_high, "negative revolution phase [10/3 ns]", "counts");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step3b_Fit/SeeTheFit/the_fit_%i_%s", i, channelNames[i].c_str());
		sprintf(tmpTitle, "the_fit_%i_%s",i,channelNames[i].c_str());
		theFit[i] = MakeTH1('D', tmpPath, tmpTitle, tofit_div, tofit_low,
				tofit_high, "negative revolution phase [10/3 ns]", "counts");
	}

	fitTime = 0;
	lastFitTime = 0;
}

//-----------------------------------------------------------
TLda3bBunchFitProc::~TLda3bBunchFitProc() {
	cout << "**** TLda3bBunchFitProc: Delete" << endl;

}
//-----------------------------------------------------------
Bool_t TLda3bBunchFitProc::BuildEvent(TGo4EventElement* dest) {
	Bool_t isValid = kFALSE; // validity of output event
	TLdaAnlEvent* in_evt =
			dynamic_cast<TLdaAnlEvent*> (GetInputEvent("IntDiff"));
	TLda3bBunchFitEvent* out_evt = (TLda3bBunchFitEvent*) dest;
	out_evt->eventMbsTime = in_evt->eventMbsTime;//in absolute time
	out_evt->eventUsefulLifeTime = in_evt->eventUsefulLifeTime;//external clock in during good measuring conditions
	// laser not switched, event empty
	if ((in_evt == 0) || !in_evt->IsValid() || (fPar2->flagBunchFit == 0)) { // input invalid
		out_evt->SetValid(isValid); // invalid
		return isValid; // must be same is for SetValid
	}
	fittingCtr++;
	fitTime = in_evt->whenSwitched;
	fitTimeHist->Fill(in_evt->stepNr+1, fitTime - lastFitTime);//take care: Bin=0 is underflow bin, Bin=1 is first data bin
	fitLifeTimeHist->Fill(in_evt->stepNr+1, in_evt->eventUsefulLifeTime);
	//	if (in_evt->phaseCounts[PMTs[pmtIndex]]=!0) {
	cout << "BunchFit: Begin " << fittingCtr << ". Fit. "<<"we are in "<<in_evt->stepNr<<". step from input event"<<endl;
	//-----------Start fitting here----------------

	//	TApplication theApp("Application", 0, 0);
	//	BunchFit();
	//	theApp.Run();

	// create fitter, select fit function and add standard actions list
	//	   TGo4Fitter fitter("Fitter", TGo4Fitter::ff_ML_Poisson, kTRUE);

	// add histogram to fitter, which should be fitted


	Double_t fitContentChecker;
	Double_t binContent;
	//		fill histogram from TArrayI
	//		for (int i=0;i<MAX_TDC;i++){
	//		in_evt->phaseCounts[i].Copy(toBeFitted[i]);// Dangerous! More care to be taken about clearing up!
	//		}
	for (UInt_t pmtIndex = 0; pmtIndex < PMT_NUMBER; pmtIndex++) {// fit only the provided list of  PMTs
		fitContentChecker = 0;
		binContent = 0;
		//dont use the border area of the histogram
		for (Int_t i = 1; i < floor((fPar->period)) - 1; i++) {//take care: excludes the the upper bound for i
			binContent = in_evt->phaseCounts[PMTs[pmtIndex]][i];
			toBeFitted[PMTs[pmtIndex]]->SetBinContent(i + 1, binContent);//take more care: i=0 is underflow bin, i=1 is first data bin
			fitContentChecker += binContent;
		}
		if (fitContentChecker > 100) {// only if enough hits
			TGo4Fitter fitter("Fitter", 1, kTRUE);
			//start with the right peak,which should be the reference peak
			fitter.AddH1("dataReference", toBeFitted[PMTs[pmtIndex]], kFALSE,
					floor(fPar->period / 2) + 1, floor(fPar->period) - 2);//take care: probably includes the rrange-bin!
			fitter.AddPolynomX("dataReference", "Pol", 0);
			fitter.AddModel("dataReference",
					new TFoldGauss2("TwoFoldGauss", 6, kFALSE));
			//			fitter.AddModel("dataReference",
			//								new TFoldGauss1("FoldGaussShort", 3, kTRUE));
			//			fitter.AddGauss1("dataReference", "Gauss1", 160, 10);
			//		fitter.SetH1(tmpString, toBeFitted[ch], kFALSE);//Hier ist wichtig, dass kFALSE steht
			// set initial values of user model parameters
			//			   fitter.SetParValue("TwoFoldGauss.Ampl", 100.);

			fitter.SetParValue("TwoFoldGauss.Par0", 50.);//amp1
			fitter.SetParRange("TwoFoldGauss.Par0", 0.001, 1000000.);
			fitter.SetParValue("TwoFoldGauss.Par1", 50.);//amp2
			fitter.SetParRange("TwoFoldGauss.Par1", 0.001, 1000000.);
			fitter.SetParValue("TwoFoldGauss.Par2", 122.);//pos
			fitter.SetParRange("TwoFoldGauss.Par2", 100.,
					140.);
			fitter.SetParValue("TwoFoldGauss.Par3", 3.2);//sigma
			fitter.SetParRange("TwoFoldGauss.Par3", 2.8, 8);
			fitter.SetParValue("TwoFoldGauss.Par4", 1 / 20.);//lambda1=1/tau1
			fitter.SetParRange("TwoFoldGauss.Par4", 0.04, 1 / 15.);
			fitter.SetParValue("TwoFoldGauss.Par5", 1 / 1.2);//lambda2=1/tau2
			fitter.SetParRange("TwoFoldGauss.Par5", 1 / 1.5, 1 / 0.9);

			// execute all actions
			fitter.DoActions();

			TH1* refHist = (TH1*) fitter.CreateDrawObject("GraphOfRefFit",
					"dataReference", kTRUE);
			theFit[PMTs[pmtIndex]]->Reset();
			theFit[PMTs[pmtIndex]]->Add(refHist);
			delete refHist;


			//			Double_t peakArea =
			//					(Double_t) fitter.GetParValue("TwoFoldGauss.Par0");//function is already normalized
			//							* (Double_t) fitter.GetParValue("TwoFoldGauss.Par1")
			//							/ (Double_t) sqrt(2 * (Double_t) 3.14159265);
			Double_t ampLong = (Double_t) fitter.GetParValue(
					"TwoFoldGauss.Par0");
			fitAmpLong[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr, ampLong);//should be equal to count area of fit graph
			out_evt->amplitudeLong[PMTs[pmtIndex]] = ampLong;
			Double_t ampShort = (Double_t) fitter.GetParValue(
					"TwoFoldGauss.Par1");
			fitAmpShort[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr, ampShort);
			out_evt->amplitudeShort[PMTs[pmtIndex]] = ampShort;
			Double_t backgrd = (Double_t) fitter.GetParValue("Pol_0.Ampl")
					* fPar->period;
			uniformBackground[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr,
					backgrd);
			out_evt->background[PMTs[pmtIndex]] = backgrd;

			Double_t referenceParameters[fitter.NumPars()];
			fitter.GetParsValues(referenceParameters);
			//			referenceParameters[2] -= (fPar->period / 2);

			fitter.RemoveData("dataReference");
			//			fitter.RemoveModel("TwoFoldGauss");
			//			fitter.RemoveModel("Pol");
			//
			//continue with the left peak, which should be the signal peak
			fitter.AddH1("dataMeasurement", toBeFitted[PMTs[pmtIndex]], kFALSE,
					1, floor(fPar->period / 2) - 1);//take care: probably includes the rrange-bin!
			fitter.AddPolynomX("dataMeasurement", "Pol", 0);
			fitter.AddModel("dataMeasurement",
					new TFoldGauss2("TwoFoldGauss", 6, kFALSE));

			// leave all parameters the same, shift only the peak position
			fitter.SetParsValues(referenceParameters);
			fitter.SetParRange("TwoFoldGauss.Par2", 0, 10000);
			Double_t refPos = fitter.GetParValue("TwoFoldGauss.Par2");
			Double_t measPos = refPos - fPar->period / 2;
			fitter.SetParValue("TwoFoldGauss.Par2", measPos);//pos

			fitter.SetParFixed("Pol1_0.Ampl", kTRUE);
			fitter.SetParFixed("TwoFoldGauss.Par0", kTRUE);
			fitter.SetParFixed("TwoFoldGauss.Par1", kTRUE);
			fitter.SetParFixed("TwoFoldGauss.Par2", kTRUE);
			fitter.SetParFixed("TwoFoldGauss.Par3", kTRUE);
			fitter.SetParFixed("TwoFoldGauss.Par4", kTRUE);
			fitter.SetParFixed("TwoFoldGauss.Par5", kTRUE);

			Double_t width = fitter.GetParValue("TwoFoldGauss.Par3");

			fitter.AddGauss1("dataMeasurement", "fluorescenceGauss", measPos,
					width, 1, 0);
			fitter.SetParFixed("fluorescenceGauss.Pos", kTRUE);
			fitter.SetParFixed("fluorescenceGauss.Width", kTRUE);
			// execute all actions
			fitter.DoActions();
			Double_t ampSig = (Double_t) fitter.GetParValue("fluorescenceGauss.Ampl")*width*TMath::Sqrt(2.*TMath::Pi());//should be equal to count area of additional Gauss
			fitAmpSignal[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr, ampSig);
			out_evt->signal[PMTs[pmtIndex]] = ampSig;
			cout << "fitted channel: " << PMTs[pmtIndex] << ",";
			if (in_evt->stepNr < 3 && PMTs[pmtIndex] == MAINPMT_CH) {//only for a few cases print/draw result
				//				fitter.Draw("#GraphOfFit,TwoFoldGauss");//this analysis is in batch mode, so this root command has no effect.
				fitter.Print("Pars");
			}
			TH1* measHist = (TH1*) fitter.CreateDrawObject("GraphOfMeasFit",
					"dataMeasurement", kTRUE);
			theFit[PMTs[pmtIndex]]->Add(measHist);
			delete measHist;

		} else {
			fitAmpLong[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr, 0);
			fitAmpShort[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr, 0);
			uniformBackground[PMTs[pmtIndex]]->SetBinContent(in_evt->stepNr, 0);
		}

	}
	lastFitTime = fitTime;

	//
	//	//-----------End fitting here----------------

	// see comments in UnpackProc
	isValid = kTRUE;
	out_evt->SetValid(isValid);
	return isValid;
}

