// $Id: TLda4WaveMergProc.cxx 520 2009-11-10 16:15:53Z goofy $
//-----------------------------------------------------------------------
//       The GSI Online Offline Object Oriented (Go4) Project
//         Experiment Data Processing at EE department, GSI
//-----------------------------------------------------------------------
// Copyright (C) 2000- GSI Helmholtzzentrum fr Schwerionenforschung GmbH
//                     Planckstr. 1, 64291 Darmstadt, Germany
// Contact:            http://go4.gsi.de
//-----------------------------------------------------------------------
// This software can be used under the license agreements as stated
// in Go4License.txt file which is part of the distribution.
//-----------------------------------------------------------------------

#include "TLda4WaveMergProc.h"

#include <math.h>

#include "TH1.h"
#include "TH2.h"
#include "Riostream.h"
#include "TROOT.h"

#include "TGo4WinCond.h"
#include "TGo4Analysis.h"

#include "TGo4Fitter.h"

#include "TLda3aIntDiffEvent.h"
#include "TLda3bBunchFitEvent.h"
#include "TLda4WaveMergEvent.h"
#include "TLdaParameters.h"

//-----------------------------------------------------------
TLda4WaveMergProc::TLda4WaveMergProc() :
		TGo4EventProcessor() {
}
//-----------------------------------------------------------
TLda4WaveMergProc::TLda4WaveMergProc(const char* name) :
		TGo4EventProcessor(name) {
	cout << "**** TLda4WaveMergProc: Create" << endl;
	//// init user analysis objects:
	// if unpack was enabled, parameters have been printed already
	//   if(TGo4Analysis::Instance()->GetAnalysisStep("Unpack")->IsProcessEnabled())
	//	    gROOT->ProcessLine(".x setparam.C(0)");
	//   else gROOT->ProcessLine(".x setparam.C(1)");

	//This is our parameter bitch! Loaded here for scope reasons

	fPar = (TLdaParameters *) GetParameter("TimeParameters");
	fPar2 = (Parameter2ndList *) GetParameter("ControlParameters");

	char tmpPath[100];
	char tmpTitle[100];

	//puuuh, wie geht das?
	Int_t scan_div = 201;
	Double_t scan_low = -0.5;
	Double_t scan_high = 200.5;

	sprintf(tmpPath, "Step4_DataShow/LifeTimeCurr");
	sprintf(tmpTitle, "LifeTimeCurr");
	lifeCurr = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low, scan_high,
			"step no.", "Life time per bin in curr scan [s]");

	sprintf(tmpPath, "Step4_DataShow/LifeTimeAcc");
	sprintf(tmpTitle, "LifeTimeAcc");
	lifeAcc = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low, scan_high,
			"step no.", "Life time per bin accumulated [s]");

	//_______________________first intDiff results as function of wavelength__________________

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath,
				"Step4_DataShow/CurrentScan/IntDiff/IntDiffCurrScan_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "IntDiffCurrentScan_%i_%s", i,
				channelNames[i].c_str());
		intDiffScan[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Diff. cts in peaks meas. - ref.");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step4_DataShow/Accumulate/IntDiff/IntDiffAcc_%i_%s",
				i, channelNames[i].c_str());
		sprintf(tmpTitle, "IntDiffAcc_%i_%s", i, channelNames[i].c_str());
		intDiffAcc[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Acc. Diff. cts in peaks meas. - ref.");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step4_DataShow/AccNorm/IntDiff/IntDiffAccNorm_%i_%s",
				i, channelNames[i].c_str());
		sprintf(tmpTitle, "IntDiffAccNorm_%i_%s", i, channelNames[i].c_str());
		intDiffAccNorm[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.",
				"Acc. Diff. cts per 10s lifetime meas. - ref.");
	}

	for (Int_t i = 0; i < MAX_TDC; i++) {
		intDiffSig[i].Set(scan_div);//this subarray is automatically filled with zeros
		intDiffSigNorm[i].Set(scan_div);
	}

	//_______________________intTotal results as function of wavelength__________________

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath,
				"Step4_DataShow/CurrentScan/IntTotal/IntTotalCurrScan_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "IntTotalCurrentScan_%i_%s", i,
				channelNames[i].c_str());
		intTotalScan[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Total. cts in peaks meas. - ref.");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step4_DataShow/Accumulate/IntTotal/IntTotalAcc_%i_%s",
				i, channelNames[i].c_str());
		sprintf(tmpTitle, "IntTotalAcc_%i_%s", i, channelNames[i].c_str());
		intTotalAcc[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Acc. Total. cts in peaks meas. - ref.");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath,
				"Step4_DataShow/AccNorm/IntTotal/IntTotalAccNorm_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "IntTotalAccNorm_%i_%s", i, channelNames[i].c_str());
		intTotalAccNorm[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.",
				"Acc. Total. cts per 10s lifetime meas. - ref.");
	}

	for (Int_t i = 0; i < MAX_TDC; i++) {
		intTotalSig[i].Set(scan_div);//this subarray is automatically filled with zeros
		intTotalSigNorm[i].Set(scan_div);
	}

	//_______________________now fit results as function of wavelength__________________

	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step4_DataShow/CurrentScan/fit/fitCurrScan_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "fitCurentScan_%i_%s", i, channelNames[i].c_str());
		fitScan[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Diff. cts in peaks meas. - ref.");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step4_DataShow/Accumulate/fit/fitAcc_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "fitAcc_%i_%s", i, channelNames[i].c_str());
		fitAcc[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Acc. Diff. cts in peaks meas. - ref.");
	}
	for (Int_t i = 0; i < MAX_TDC; i++) {
		sprintf(tmpPath, "Step4_DataShow/AccNorm/fit/fitAccNorm_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "fitAccNorm_%i_%s", i, channelNames[i].c_str());
		fitAccNorm[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.",
				"Acc. Diff. cts per 10s lifetime meas. - ref.");
	}

	for (Int_t i = 0; i < MAX_TDC; i++) {
		fitSig[i].Set(scan_div);//this subarray is automatically filled with zeros
		fitSigNorm[i].Set(scan_div);
	}

	//_______________________now scaler as function of wavelength__________________

	for (Int_t i = 0; i < MAX_SCA; i++) {
		sprintf(tmpPath, "Step4_DataShow/CurrentScan/scaler/scCurrScan_%i_%s",
				i, channelNames[i].c_str());
		sprintf(tmpTitle, "scalerCurentScan_%i_%s", i, channelNames[i].c_str());
		scalerScan[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "counts in this step");
	}

	scalerG1minG3Scan = MakeTH1('I',
			"Step4_DataShow/CurrentScan/scaler/scCurrScan_G1minG3",
			"scalerCurentScan_G1minG3", scan_div, scan_low, scan_high,
			"step no.", "subtracted counts in this step");

	for (Int_t i = 0; i < MAX_SCA; i++) {
		sprintf(tmpPath, "Step4_DataShow/Accumulate/scaler/scAcc_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "scalerAcc_%i_%s", i, channelNames[i].c_str());
		scalerAcc[i] = MakeTH1('I', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "subtracted accumulated cps");
	}

	scalerG1minG3Acc = MakeTH1('I',
			"Step4_DataShow/Accumulate/scaler/scAcc_G1minG3",
			"scalerAcc_G1minG3", scan_div, scan_low, scan_high, "step no.",
			"subtravted, accumulated cps");

	for (Int_t i = 0; i < MAX_SCA; i++) {
		sprintf(tmpPath, "Step4_DataShow/AccNorm/scaler/scAccNorm_%i_%s", i,
				channelNames[i].c_str());
		sprintf(tmpTitle, "scalerAccNorm_%i_%s", i, channelNames[i].c_str());
		scalerAccNorm[i] = MakeTH1('D', tmpPath, tmpTitle, scan_div, scan_low,
				scan_high, "step no.", "Acc. counts per 10s lifetime");
	}

	scalerG1minG3AccNorm = MakeTH1('D',
			"Step4_DataShow/AccNorm/scaler/scAccNorm_G1minG3",
			"scalerAccNorm_G1minG3", scan_div, scan_low, scan_high, "step no.",
			"subtracted Acc. counts per 10s lifetime");

	for (Int_t i = 0; i < MAX_SCA; i++) {
		scalerSig[i].Set(scan_div);	//this subarray is automatically filled with zeros
		scalerSigNorm[i].Set(scan_div);
	}
	//______________________________________----___________________

	stepToBinNo.Set(scan_div);
	for (int i = 0; i < scan_div; i++) {
		stepToBinNo[i] = i + 1;
	}

	usefulLifeTime.Set(scan_div);
}

//-----------------------------------------------------------
TLda4WaveMergProc::~TLda4WaveMergProc() {
	cout << "**** TLda4WaveMergProc: Delete" << endl;
	//we don't want the signal to be deleted!
	outFile.close();
//	for(int i=0; i<MAX_TDC; i++){
//		intDiffSig[i].Set(0);
//		intDiffSigNorm[i].Set(0);
//	}
//	wavelength.Set(0);
}
//-----------------------------------------------------------
Bool_t TLda4WaveMergProc::BuildEvent(TGo4EventElement* dest) {
	Bool_t isValid = kFALSE; // validity of output event
	TLda3aIntDiffEvent* in_evt =
			dynamic_cast<TLda3aIntDiffEvent*>(GetOutputEvent("IntDiff"));
	TLda3bBunchFitEvent* in_evt2 =
			dynamic_cast<TLda3bBunchFitEvent*>(GetOutputEvent("BunchFit"));
	TLda4WaveMergEvent* out_evt = (TLda4WaveMergEvent*) dest;

	iIon = 0;

	if (in_evt == 0) { // input invalid
		cout << "Programming error - event TLda3aIntDiffEvent not found"
				<< endl;
		out_evt->SetValid(isValid); // invalid
		return isValid;
	}

	if (in_evt2 == 0) { // input invalid
		cout << "Programming error - event TLda3bBunchFitEvent not found"
				<< endl;
		out_evt->SetValid(isValid); // invalid
		return isValid;
	}

	//event 3a invalid
	if (!in_evt->IsValid()) { // input invalid
		out_evt->SetValid(isValid); // invalid
		return isValid;
	}

	//no useful data in this event!
	if (in_evt->eventUsefulLifeTime == 0) {
		out_evt->SetValid(isValid);
		return isValid;
	}

	//step Nr in TLdaAnl also accepted if it is 0
	Int_t step = in_evt->stepNr;
//	cout<<"stepNr in Merg aus in_evt="<<in_evt->stepNr;

	usefulLifeTime[step] += in_evt->eventUsefulLifeTime; // in sec
	cout << endl;
	cout << "WaveMerge: useful time in this event = "
			<< in_evt->eventUsefulLifeTime;

	lifeCurr->SetBinContent(stepToBinNo[step], in_evt->eventUsefulLifeTime);
	lifeAcc->Fill(step, in_evt->eventUsefulLifeTime); // Attention: when filling don't enter bin number but wavelength
	iIon = ((in_evt->lasingAcc[DCTRAFO_CH] / in_evt->eventUsefulLifeTime)
			- DCTRAFO_OFFSET) / DCTRAFO_PROP;
	//Fill the arrays
	for (Int_t i = 0; i < MAX_TDC; i++) {
		intDiffSig[i][step] += in_evt->signal[i];
		//normalize to 10s lifetime
		intDiffSigNorm[i][step] = intDiffSig[i][step] * 10
				/ usefulLifeTime[step];
		intTotalSig[i][step] += in_evt->signalAndBackground[i];
		//normalize to 10s lifetime
		intTotalSigNorm[i][step] = intTotalSig[i][step] * 10
				/ usefulLifeTime[step];

	}

//	//readout the hardware gated stuff; before this was done by evil kanevil hack
//	hardwareGateSignal=(Int_t)in_evt->lasingAcc[13];
//	hardwareGateBackground=(Int_t)in_evt->lasingAcc[12];
//	//export data to outFile

	//Bool_t newFile = TGo4Analysis::Instance()->IsNewInputFile();
	//|| newFile
	if (!outFile.is_open()) {//not working yet, using all the time the same outFile
		char inFileName[100];
		sprintf(inFileName, in_evt->lmdName);
		char outFileName[100];
		char lmdIndex[30];
		Int_t nrDigits = 4;
		Int_t skipChar = 4;
		// snprintf(outFileName, sizeof(outFileName), "result%03d.txt", i);
		memset(lmdIndex, 0, sizeof(lmdIndex));
		for (int i = 0; i < nrDigits; i++) {
			lmdIndex[i] = inFileName[i
					+ (strlen(inFileName) - nrDigits - skipChar)];//copy the nrDigits digits to the index. skipChar in the end are the ".lmd"
		}
		//		inFileName.copy(lmdIndex,10,inFileName.size()-10);
		lmdIndex[nrDigits] = 0;
		sprintf(outFileName, "result%s%s", lmdIndex, ".txt");
		cout << "open file " << outFileName << endl;
		outFile.open(outFileName);
		outFile << "Linux_time_[s]" << "\t" << "step" << "\t" << "setWL[nm]"
				<< "\t" << "Atos avg [nm]" << "\t" << "last Atos [nm]" << "\t"
				<< "lifeTime [s]" << "\t" << "DC-Trafo[mA]" << "\t"
				<< "intDiff[0]" << "\t" << "intDiff[1]" << "\t" << "intDiff[2]"
				<< "\t" << "intDiff[3]" << "\t" << "intDiff[4]" << "\t"
				<< "intDiff[5]" << "\t" << "intDiff[6]"
//				<< "\t" << "intDiff[hardwareGate]"
				<< "\t" << "total[0]" << "\t" << "total[1]" << "\t"
				<< "total[2]" << "\t" << "total[3]" << "\t" << "total[4]"
				<< "\t" << "total[5]" << "\t" << "total[6]";
		for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
			outFile << "\t" << "LV " << lvNames[i].c_str();
		}
		outFile << "\n";
//				<< "\t" << "total[hardwareGate]"<< "\n";

		//		string::iterator it;
		//		for (it = str.begin(); it < str.end(); it++){
		//			cout << *it;
		//		}

	}
	outFile << fixed;
	outFile << (Double_t) in_evt->eventMbsTime << "\t" << step << "\t"
			<< in_evt->cobraSetWL << "\t" << in_evt->atosAvgWL << "\t"
			<< in_evt->lastAtosWL << "\t" << in_evt->eventUsefulLifeTime << "\t"
			<< iIon << "\t" << (Int_t) in_evt->signal[0] << "\t"
			<< (Int_t) in_evt->signal[1] << "\t" << (Int_t) in_evt->signal[2]
			<< "\t" << (Int_t) in_evt->signal[3] << "\t"
			<< (Int_t) in_evt->signal[4] << "\t" << (Int_t) in_evt->signal[5]
			<< "\t" << (Int_t) in_evt->signal[6]

			<< "\t" << (Int_t) in_evt->signalAndBackground[0] << "\t"
			<< (Int_t) in_evt->signalAndBackground[1] << "\t"
			<< (Int_t) in_evt->signalAndBackground[2] << "\t"
			<< (Int_t) in_evt->signalAndBackground[3] << "\t"
			<< (Int_t) in_evt->signalAndBackground[4] << "\t"
			<< (Int_t) in_evt->signalAndBackground[5] << "\t"
			<< (Int_t) in_evt->signalAndBackground[6];
	for (int i = 0; i < TOTAL_LV_NUMBERS; i++) {
		outFile << "\t" << in_evt->lvAvgVal[i];
	}

	outFile << "\n";
	//outFile.close();
	//Wrong input event! this has to be 2!
	/*	for(Int_t i=0; i<MAX_TDC; i++){
	 fitSig[i][step] += in_evt->signal[i];
	 //normalize to 10s lifetime
	 fitSigNorm[i][step] = fitSig[i][step]*10/usefulLifeTime[step];
	 }*/

	for (Int_t i = 0; i < MAX_TDC; i++) {
		fitSig[i][step] += in_evt2->signal[i];
		//normalize to 10s lifetime
		fitSigNorm[i][step] = fitSig[i][step] * 10 / usefulLifeTime[step];
	}

	for (Int_t i = 0; i < MAX_SCA; i++) {
		scalerSig[i][step] += in_evt->lasingAcc[i];
		//normalize to 10s lifetime
		scalerSigNorm[i][step] = scalerSig[i][step] * 10 / usefulLifeTime[step];
	}

	cout << "intDiffSig[ch=0][step=" << step << "] = " << intDiffSig[0][step]
			<< ", accumulated usefulLifeTime[step] = " << usefulLifeTime[step];
	cout << ", fitSig[ch=][Step] = " << fitSig[0][step] << endl;
	//Fill the histograms
	for (int i = 0; i < MAX_TDC; i++) {
		intDiffScan[i]->SetBinContent(stepToBinNo[step], in_evt->signal[i]);//take more care: bin=0 is underflow bin, bin=1 is first data bin
		intDiffAcc[i]->Fill(step, in_evt->signal[i]);// Attention: when filling don't enter bin number but wavelength
		intDiffAccNorm[i]->SetBinContent(stepToBinNo[step],
				10 * intDiffSig[i][step] / usefulLifeTime[step]);
	}

	for (int i = 0; i < MAX_TDC; i++) {
		intTotalScan[i]->SetBinContent(stepToBinNo[step],
				in_evt->signalAndBackground[i]);//take more care: bin=0 is underflow bin, bin=1 is first data bin
		intTotalAcc[i]->Fill(step, in_evt->signalAndBackground[i]);	// Attention: when filling don't enter bin number but wavelength
		intTotalAccNorm[i]->SetBinContent(stepToBinNo[step],
				10 * intTotalSig[i][step] / usefulLifeTime[step]);
	}

	for (int i = 0; i < MAX_TDC; i++) {
		fitScan[i]->SetBinContent(stepToBinNo[step], in_evt2->signal[i]);//take more care: bin=0 is underflow bin, bin=1 is first data bin
		fitAcc[i]->Fill(step, in_evt2->signal[i]);// Attention: when filling don't enter bin number but wavelength
		fitAccNorm[i]->SetBinContent(stepToBinNo[step],
				10 * fitSig[i][step] / usefulLifeTime[step]);
	}

	for (int i = 0; i < MAX_SCA; i++) {
		scalerScan[i]->SetBinContent(stepToBinNo[step], in_evt->lasingAcc[i]);//take more care: bin=0 is underflow bin, bin=1 is first data bin
		scalerAcc[i]->Fill(step, in_evt->lasingAcc[i]);	// Attention: when filling don't enter bin number but wavelength
		scalerAccNorm[i]->SetBinContent(stepToBinNo[step],
				10 * scalerSig[i][step] / usefulLifeTime[step]);
	}

	Long64_t diffForCoasting = in_evt->lasingAcc[CU_GATED1_CH]
			- in_evt->lasingAcc[CU_GATED3_CH];

	scalerG1minG3Scan->SetBinContent(stepToBinNo[step], diffForCoasting);
	scalerG1minG3Acc->Fill(step, diffForCoasting);
	scalerG1minG3AccNorm->SetBinContent(stepToBinNo[step],
			10 * (scalerSig[CU_GATED1_CH][step] - scalerSig[CU_GATED3_CH][step])
					/ usefulLifeTime[step]);
//	//loop finished, ship and clean// passiert jetzt wohl nicht mehr. step 4 braucht kein out event
//	if(step == 150){
//		//set scan data
//	    out_evt->stepWL = fPar->stepWL;
//	    out_evt->startWL = fPar->startWL;
//	    out_evt->stepNr = fPar->stepNr;
//	    out_evt->eventMbsTime = in_evt->eventMbsTime;
//	    out_evt->eventUsefulLifeTime = in_evt->eventUsefulLifeTime;
//		isValid = kTRUE;
//	}

	if (!in_evt2->IsValid()) { // input invalid
		out_evt->SetValid(isValid); // invalid
		return isValid;
	}

	// out
	out_evt->SetValid(isValid);
	return isValid;
}

